package org.openxma.addons.ui.table.caller.mdl;

import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.addons.ui.table.caller.mdl.dto.CallerDataView;
import org.openxma.addons.ui.table.caller.mdl.dto.CallerView;
import org.openxma.addons.ui.table.caller.mdl.model.CallerData;
import org.openxma.dsl.platform.assembler.DtoAssembler;
import org.openxma.dsl.platform.assembler.DtoAssemblerFactory;
import org.openxma.dsl.platform.assembler.DtoAssemblerRegistry;
import org.openxma.dsl.platform.converter.Converter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;

public class DtoAssemblerFactoryMdlGen implements DtoAssemblerFactory {

	private List<DtoAssembler<?>> dtoAssemblers;

	@Autowired
    private DtoAssemblerRegistry dtoAssemblerRegistry;
    
    @Autowired
    private Converter converter;

    public DtoAssemblerFactoryMdlGen() {
		dtoAssemblers = new ArrayList<DtoAssembler<?>>(2);
		initDtoAssemblers();
	}


	public List<DtoAssembler<?>> getDtoAssemblers() {
		return dtoAssemblers;
	}

	protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
		return dtoAssemblerRegistry;
	}
	
	protected Converter getConverter() {
		return converter;
	}

	private void initDtoAssemblers() {
		dtoAssemblers.add(createDtoAssemblerCallerDataView());
		dtoAssemblers.add(createDtoAssemblerCallerView());
	}


	private DtoAssembler<CallerDataView> createDtoAssemblerCallerDataView() {
		return new DtoAssembler<CallerDataView>() {
		    public CallerDataView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof CallerData) {
					CallerData callerData = (CallerData) sourceEntity;
		    		CallerDataView result = mapFromCallerDataToCallerDataView(callerData);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public CallerDataView mapFromEntity(Object sourceEntity, CallerDataView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof CallerData) {
					CallerData callerData = (CallerData) sourceEntity;
		    		mapFromCallerDataToCallerDataView(callerData, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(CallerDataView sourceDto, Class<T> target) {
				if (CallerDataView.class.equals(target)) {
					CallerDataView result = new CallerDataView();
					mapFromCallerDataViewToCallerDataView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (CallerData.class.equals(target)) {
		    		return target.cast(mapFromCallerDataViewToCallerData(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(CallerDataView sourceDto, Object targetObject) {
		    	if (targetObject instanceof CallerDataView) {
					mapFromCallerDataViewToCallerDataView(sourceDto, (CallerDataView) targetObject);
				}				
		    	else if (targetObject instanceof CallerData) {
		    		mapFromCallerDataViewToCallerData(sourceDto, (CallerData)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromCallerDataViewToCallerDataView(CallerDataView sourceDto, CallerDataView callerDataView) {
   		callerDataView.setOid(sourceDto.getOid());
   		callerDataView.setDatCaller(sourceDto.getDatCaller());
   		callerDataView.setNumCaller(sourceDto.getNumCaller());
   		callerDataView.setCodCaller(sourceDto.getCodCaller());
    }
    
    protected CallerData mapFromCallerDataViewToCallerData(CallerDataView sourceDto) {
    	return mapFromCallerDataViewToCallerData(sourceDto, MapperContextHolder.createEntity(CallerData.class));
    }

    protected CallerData mapFromCallerDataViewToCallerData(CallerDataView sourceDto, CallerData callerData) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return callerData;
    	}
	    MapperContextHolder.getContext().put(sourceDto, callerData);
		if (sourceDto.getOid() != null) {
			callerData.setOid(sourceDto.getOid());
		}
		callerData.setDatCaller(sourceDto.getDatCaller());
		callerData.setNumCaller(sourceDto.getNumCaller());
		callerData.setCodCaller(sourceDto.getCodCaller());
	    return callerData;
	}
	
	protected CallerDataView mapFromCallerDataToCallerDataView(CallerData callerData) {
		if (MapperContextHolder.getContext().containsKey(callerData)) {
			return (CallerDataView) MapperContextHolder.getContext().get(callerData);
		}
		CallerDataView callerDataView = new CallerDataView();
    	MapperContextHolder.getContext().put(callerData, callerDataView);
		callerDataView.setOid(callerData.getOid());
		callerDataView.setDatCaller(callerData.getDatCaller());
		callerDataView.setNumCaller(callerData.getNumCaller());
		callerDataView.setCodCaller(callerData.getCodCaller());
        return callerDataView;
	}
	
	protected CallerDataView mapFromCallerDataToCallerDataView(CallerData callerData, CallerDataView callerDataView) {
		if (MapperContextHolder.getContext().containsKey(callerData)) {
			return callerDataView;
		}		
    	MapperContextHolder.getContext().put(callerData, callerDataView);
		callerDataView.setOid(callerData.getOid());
		callerDataView.setDatCaller(callerData.getDatCaller());
		callerDataView.setNumCaller(callerData.getNumCaller());
		callerDataView.setCodCaller(callerData.getCodCaller());
        return callerDataView;
	}
	
	

	private DtoAssembler<CallerView> createDtoAssemblerCallerView() {
		return new DtoAssembler<CallerView>() {
		    public CallerView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof CallerData) {
					CallerData callerData = (CallerData) sourceEntity;
		    		CallerView result = mapFromCallerDataToCallerView(callerData);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public CallerView mapFromEntity(Object sourceEntity, CallerView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof CallerData) {
					CallerData callerData = (CallerData) sourceEntity;
		    		mapFromCallerDataToCallerView(callerData, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(CallerView sourceDto, Class<T> target) {
				if (CallerView.class.equals(target)) {
					CallerView result = new CallerView();
					mapFromCallerViewToCallerView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (CallerData.class.equals(target)) {
		    		return target.cast(mapFromCallerViewToCallerData(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(CallerView sourceDto, Object targetObject) {
		    	if (targetObject instanceof CallerView) {
					mapFromCallerViewToCallerView(sourceDto, (CallerView) targetObject);
				}				
		    	else if (targetObject instanceof CallerData) {
		    		mapFromCallerViewToCallerData(sourceDto, (CallerData)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromCallerViewToCallerView(CallerView sourceDto, CallerView callerView) {
   		callerView.setCodCaller(sourceDto.getCodCaller());
   		callerView.setDatCaller(sourceDto.getDatCaller());
   		callerView.setNumCaller(sourceDto.getNumCaller());
    }
    
    protected CallerData mapFromCallerViewToCallerData(CallerView sourceDto) {
    	return mapFromCallerViewToCallerData(sourceDto, MapperContextHolder.createEntity(CallerData.class));
    }

    protected CallerData mapFromCallerViewToCallerData(CallerView sourceDto, CallerData callerData) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return callerData;
    	}
	    MapperContextHolder.getContext().put(sourceDto, callerData);
		callerData.setCodCaller(sourceDto.getCodCaller());
		callerData.setDatCaller(sourceDto.getDatCaller());
		callerData.setNumCaller(sourceDto.getNumCaller());
	    return callerData;
	}
	
	protected CallerView mapFromCallerDataToCallerView(CallerData callerData) {
		if (MapperContextHolder.getContext().containsKey(callerData)) {
			return (CallerView) MapperContextHolder.getContext().get(callerData);
		}
		CallerView callerView = new CallerView();
    	MapperContextHolder.getContext().put(callerData, callerView);
		callerView.setCodCaller(callerData.getCodCaller());
		callerView.setDatCaller(callerData.getDatCaller());
		callerView.setNumCaller(callerData.getNumCaller());
        return callerView;
	}
	
	protected CallerView mapFromCallerDataToCallerView(CallerData callerData, CallerView callerView) {
		if (MapperContextHolder.getContext().containsKey(callerData)) {
			return callerView;
		}		
    	MapperContextHolder.getContext().put(callerData, callerView);
		callerView.setCodCaller(callerData.getCodCaller());
		callerView.setDatCaller(callerData.getDatCaller());
		callerView.setNumCaller(callerData.getNumCaller());
        return callerView;
	}
	
	

}
