package org.openxma.addons.ui.table.caller.mdl.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import java.util.Collection;
import org.openxma.addons.ui.table.caller.mdl.dao.CallerDao;
import org.openxma.addons.ui.table.caller.mdl.dto.CallerDataView;
import org.openxma.addons.ui.table.caller.mdl.model.CallerData;
import org.openxma.addons.ui.table.caller.mdl.service.CallerDAS;
import org.openxma.dsl.platform.query.Expression;
import org.openxma.dsl.platform.query.QueryObject;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class CallerDASGenImpl implements CallerDAS {
    protected final Log logger = LogFactory.getLog(getClass());
    @Autowired(required=false)
    protected Mapper mapper;
    protected CallerDao callerDao;
    
    @Autowired
    public void setCallerDao(CallerDao callerDao) {
        this.callerDao = callerDao;
    }
    
    @Transactional
	public CallerDataView save(CallerDataView callerDataView) {
        Assert.notNull(callerDataView, "parameter 'callerDataView' must not be null");
        CallerData callerData = this.mapper.createAndMapOne(callerDataView, CallerData.class,"saveCallerData");
        this.callerDao.saveOrUpdate(callerData);
        return this.mapper.mapOne(callerData,new CallerDataView());
    }
    
    @Transactional(readOnly = true)
	public CallerDataView loadCallerDataView(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.callerDao.load(oid),CallerDataView.class);
    }
    
    @Transactional
	public void update(CallerDataView callerDataView) {
        Assert.notNull(callerDataView, "parameter 'callerDataView' must not be null");
        CallerData mappedCallerData = this.mapper.createAndMapOne(callerDataView, CallerData.class);
        CallerData callerData = this.callerDao.load(mappedCallerData.getOid());
        this.mapper.mapOne(callerDataView, callerData,"updateCallerData");
    }
    
    @Transactional
	public void deleteCallerData(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        this.callerDao.delete(oid);
    }
        
    @Transactional
    public CallerDataView saveOrUpdate(CallerDataView callerDataView) {
        Assert.notNull(callerDataView, "parameter 'callerDataView' must not be null");
        if ( callerDataView.getOid() == null ) {
            return this.save(callerDataView);
        }
        else {
            this.update(callerDataView);
            return callerDataView;
        }
    }

    
    @Transactional(readOnly=true)
	public Collection<CallerDataView> findAllData() {
        return this.mapper.createAndMapMany(this.callerDao.findAllData(),CallerDataView.class);
    }

    @Transactional(readOnly=true)
	public Collection<CallerDataView> findAllData(Expression _filter) {
        QueryObject queryObject = QueryObject.createNamedQuery(CallerDao.FIND_ALL_DATA, _filter);
        Collection<CallerData> _result =  this.callerDao.query(queryObject);
        return this.mapper.createAndMapMany(_result,CallerDataView.class);
    }

}
