package org.openxma.addons.ui.table.customizer.caller.xma.client;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import at.spardat.xma.component.EmbeddableComponent;
import at.spardat.xma.session.XMASessionClient;

/**
 * This class implements the generic client part of CallerComponent.
 * It provides the mapping between typeIds and classnames
 * of all pages in the component.
 *
 */
abstract public class CallerComponentGen extends EmbeddableComponent implements ICallerComponent {

    /**
     * Initializes the CallerComponentGen. This method is called
     * by the XMA-Runtime.
     *
     * @param session the XMASession, this CallerComponentGen belongs to.
     */
    public CallerComponentGen (XMASessionClient session) {
        super (session, false);
    }

    /** the unique typeId for the page CallerPage */
    public static final short CallerPageClass = 0;

    /**
    * Array of the classnames of all pages in
    * this component. The typeIds are used as indices
    * into this array.
     */
    static final String[] modelClasses = {
        "CallerPage"
    };

    /**
    * Gets the short classname for the given typeId of
    * a page in this component. The short classname does
    * not contain the package.
    *
    * @param typeId the unique typeId
    * @return the short classname
     */
    static public String getShortModelClass(short typeId) {
        return modelClasses[typeId];
    }

    /**
    * Gets the full classname for the given typeId of
    * a page in this component.
    *
    * @param typeId the unique typeId
    * @return the full classname
     */
    public String getModelClass(short typeId) {
        return "org.openxma.addons.ui.table.customizer.caller.xma.client."+modelClasses[typeId];
    }

    /**
     * Invokes the CallerComponent. This method is a default
     * implementation, which invokes the first dialog page of the component.
     * Modeled init functions are called by this method.
     *
     * @param parentComposite a composite of the calling page. In most
     *     cases it will be a shell. It my be null.
     */
    public void invoke(Composite parentComposite) {
        CallerPage callerPage;
        if(parentComposite instanceof Shell) {
            callerPage = new CallerPage(this,(Shell)parentComposite);
        } else {
            callerPage = new CallerPage(this);
        }
        callerPage.invoke();
    }

    /**
     * Gives the URI of the help for this component
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this component
     */
    public String getHelpUri() {
        return "help/org/openxma/addons/ui/table/customizer/caller/xma/CallerComponent.html";
    }

}
