package org.openxma.addons.ui.table.customizer.caller.xma.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.component.*;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.paging.*;
import at.spardat.xma.mdl.simple.*;
import at.spardat.xma.mdl.table.*;
import at.spardat.enterprise.fmt.*;

/**
 * This class implements the generic client part of CallerPage.
 * It implements the widgets and models of the page.
 *
 */
public abstract class CallerPageGen extends DialogPage {
    private Locale genPageLocale = null;
    private ResourceBundle genPageMessages = null;
    Table testtableW;
    at.spardat.xma.mdl.table.TableLayoutManager testtableWLM;
    ITableWMClient testtable;
    static final int C1 = 0;
    static final int C2 = 1;
    static final int C3 = 2;
    static final int C4 = 3;
    static final int C5 = 4;
    Button TestButtonW;
    PagingControlClient pagingInstanceW;
    PagingWMClient pagingInstance;
    ISimpleWMClient customizerComponentId;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the CallerPageGen.
     * @param parent the parent page
     */
    public CallerPageGen(PageClient parent) {
        super(parent,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the CallerPageGen.
     * @param component the component this page belongs to
     */
    public CallerPageGen(ComponentClient component) {
        super(component,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the CallerPageGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     */
    public CallerPageGen(ComponentClient component, Shell parentShell) {
        super(component,parentShell,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the CallerPageGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     * @param modal the SWT-style for the desired modality
     */
    public CallerPageGen(ComponentClient component, Shell parentShell, int modal) {
        super(component,parentShell,false,modal|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }


    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        if (genPageLocale == null) {
            genPageLocale = getComponent().getContext().getLocale();
        }
        return genPageLocale;
    }

    /*
     * returns the message resource bundle used for this page
    */
    final protected ResourceBundle getGenPageMessages() {
        if (genPageMessages == null) {
            genPageMessages = ResourceBundle.getBundle("org.openxma.addons.ui.table.customizer.caller.xma.client.CallerComponent",getGenPageLocale());
        }
        return genPageMessages;
    }

    /**
     * Gives the URI of the help for this page
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this page
     */
    public String getHelpUri() {
        return "help/org/openxma/addons/ui/table/customizer/caller/xma/CallerPage.html";
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        IFmt formatter;
        testtable = new TableWMClient((short) 0,this,5,TableWMClient.S_NULL|TableWM.S_ONE_WAY);
        testtable.setColumnType(C1,Types.T_STRING);
        testtable.setColumnType(C2,Types.T_DATE);
        testtable.setColumnType(C3,Types.T_BCD);
        formatter = AStringFmt.getInstance(100);
        ((AStringFmt)formatter).setMinLen(0);
        ((TableWMClient)testtable).getColumn(C1).setFormatter(formatter);
        formatter = ADateFmt.getInstance(ADateFmt.MEDIUM, getGenPageLocale());
        ((TableWMClient)testtable).getColumn(C2).setFormatter(formatter);
        formatter = ABcdFmt.getInstance(-1, -1, ABcdFmt.THOUS_SEPS, getGenPageLocale());
        ((TableWMClient)testtable).getColumn(C3).setFormatter(formatter);
        customizerComponentId = new SimpleWMClient((short) 1, Types.T_BCD, this);
        formatter = ABcdFmt.getInstance(-1, -1, ABcdFmt.DEFAULT, getGenPageLocale());
        customizerComponentId.setFmt(formatter);
        pagingInstance = new PagingWMClient((short) 2,this,testtable);
        pagingInstance.setPagingListener(new PagingListener() {
            public void reload(int offset, int pagesize) {
                pagingInstanceWReloadEvent();
            }
        });

        widgetModels = new WModel[] { (WModel) testtable, (WModel) pagingInstance, (WModel) customizerComponentId };
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        testtable = null;
        pagingInstance = null;
        customizerComponentId = null;
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return CallerComponent.CallerPageClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public CallerComponent getTypedComponent() {
        return (CallerComponent) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        FormData data;
        Control[] tabOrder;
        EventAdapter adapter = new EventAdapter(this);
        final Scaler scaler = Scaler.getInstance(getComposite());

        final Shell shell = getShell();
        if(getDialog()==this) {
            shell.setText(getGenPageMessages().getString("CallerPage"));
            shell.setImage(getComponent().getImage("org/openxma/addons/ui/table/customizer/xma/client/images/arrow_down.png"));
            shell.addHelpListener(adapter);
        } else {
            getComposite().addHelpListener(adapter);
        }
        Composite callerPageCompW = getComposite();
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        callerPageCompW.setLayout(layout);

        testtableW = new Table(callerPageCompW,SWT.SINGLE|SWT.BORDER|SWT.FULL_SELECTION
);
        testtableW.setHeaderVisible(true);
        testtableW.setLinesVisible(true);
        testtableW.setData("WIDGET_ID","testtableW");
        testtableW.addSelectionListener(adapter);
        TableColumn C1 = new TableColumn(testtableW,SWT.LEFT);
        C1.setText(getGenPageMessages().getString("CallerPage.C1"));
        C1.addSelectionListener(adapter);
        TableColumn C2 = new TableColumn(testtableW,SWT.LEFT);
        C2.setResizable(false);
        C2.setText(getGenPageMessages().getString("CallerPage.C2"));
        C2.addSelectionListener(adapter);
        TableColumn C3 = new TableColumn(testtableW,SWT.LEFT);
        C3.setResizable(false);
        C3.setText(getGenPageMessages().getString("CallerPage.C3"));
        C3.addSelectionListener(adapter);
        TableColumn C4 = new TableColumn(testtableW,SWT.LEFT);
        C4.setResizable(false);
        C4.setText(getGenPageMessages().getString("CallerPage.C4"));
        C4.addSelectionListener(adapter);
        TableColumn C5 = new TableColumn(testtableW,SWT.LEFT);
        C5.setResizable(false);
        C5.setText(getGenPageMessages().getString("CallerPage.C5"));
        C5.addSelectionListener(adapter);
        testtableWLM = new at.spardat.xma.mdl.table.TableLayoutManager();
        testtableWLM.manageTable(testtableW,at.spardat.xma.mdl.table.TableLayoutManager.NONE);
        testtableWLM.setMinLimit(scaler.convertXToCurrent(10));
        testtableWLM.setStrechColumn(2);
        testtableWLM.setPercent(0,30.0f);
        testtableWLM.setMinWidth(1,scaler.convertXToCurrent(40));
        testtableWLM.setAbsolutWidth(1,scaler.convertXToCurrent(150));
        testtableWLM.setMinWidth(2,scaler.convertXToCurrent(40));
        testtableWLM.setAbsolutWidth(2,scaler.convertXToCurrent(80));
        testtableWLM.setAbsolutWidth(3,scaler.convertXToCurrent(40));
        testtableWLM.setAbsolutWidth(4,scaler.convertXToCurrent(40));

        TestButtonW = new Button(callerPageCompW, SWT.PUSH|SWT.CENTER);
        TestButtonW.setText(getGenPageMessages().getString("CallerPage.TestButtonW"));
        TestButtonW.setData("WIDGET_ID","TestButtonW");
        TestButtonW.addSelectionListener(adapter);

        pagingInstanceW = new PagingControlClient(callerPageCompW,getGenPageLocale(),0|PagingControlClient.SHOW_START|PagingControlClient.SHOW_BACK|PagingControlClient.SHOW_JUMP|PagingControlClient.SHOW_NEXT|PagingControlClient.SHOW_END|PagingControlClient.SHOW_INFO|PagingControlClient.SHOW_PAGESIZE|PagingControlClient.SHOW_CUSTOMIZE,SWT.BORDER);
        pagingInstanceW.setData("WIDGET_ID","pagingInstanceW");
        pagingInstanceW.addSelecionListener(adapter);
        pagingInstanceW.addModifyListener(adapter);
        pagingInstanceW.addFocusListener(adapter);


        widgets = new Control[] { testtableW, TestButtonW, pagingInstanceW };

        data = new FormData();
        data.width = scaler.convertXToCurrent(340);
        data.height = scaler.convertYToCurrent(200);
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(pagingInstanceW,scaler.convertYToCurrent(3),SWT.BOTTOM);
        testtableW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(testtableW,scaler.convertYToCurrent(3),SWT.BOTTOM);
        TestButtonW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(3));
        pagingInstanceW.setLayoutData(data);



        if(getDialog()==this) {
            shell.setTabList(new Control[] { callerPageCompW });
        }
        tabOrder = new Control[3];
        tabOrder[0] = testtableW;
        tabOrder[1] = TestButtonW;
        tabOrder[2] = pagingInstanceW;
        callerPageCompW.setTabList(tabOrder);
        if(getDialog()==this) {
            shell.setSize(scaler.convertXToCurrent(640), scaler.convertYToCurrent(480));
        }
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        testtableW = null;
        TestButtonW = null;
        pagingInstanceW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
        TableColumn[] cols;
        testtableW.setData(testtable.getUIDelegate());
        testtable.getUIDelegate().attachUI(testtableW,null);
        cols = testtableW.getColumns();
        for(int i = 0; i<cols.length; i++) {
            cols[i].setData(testtable.getUIDelegate());
        }

        pagingInstanceW.setData(pagingInstance.getUIDelegate());
        pagingInstance.getUIDelegate().attachUI(pagingInstanceW,null);
        testtableW.setData("PAGING_CONTROL",pagingInstance);
    }

    /**
     * This method will be called every time the CallerPage becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == TestButtonW) {
            TestButtonWEvent();
        } else if (event.widget == pagingInstanceW) {
            // not called by clientEvent
        }
    }

    /**
     * This method will be called every time the user clicks on TestButtonW.
     */
    abstract protected void TestButtonWEvent();

    /**
     * This method will be called every time the user clicks on a navigation button of pagingInstanceW.
     */
    abstract protected void pagingInstanceWReloadEvent();

}
