package org.openxma.addons.ui.table.customizer.common;

import java.io.Serializable;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;

public abstract class CommonDataModel implements Serializable {

    private static final long serialVersionUID = 1L;

    public enum CodDatatype {
        STRING("String"), DATE("Date"), TIMESTAMP("Timestamp"),INTEGER("Integer"), LONG("Long"), FLOAT("Float"), DOUBLE("Double"), BIGDECIMAL("BigDecimal"), DOM("Enum"), BOOL("Boolean");
        
        private String nativeName;
        private CodDatatype(String nativeName) {
            this.nativeName = nativeName;            
        }


        public static CodDatatype parse(String codDataType) {
            CodDatatype res = null;
            for (CodDatatype type : CodDatatype.values()){
                if (type.name().equals(codDataType)) {
                    return type;
                }
            }
            return res;
        }
//        public static CodDatatype xparse(short codDataType) {
//            CodDatatype res = null;
//            switch (codDataType) {
//            case Types.T_STRING:
//                res= CodDatatype.STRING;
//                break;
//            case Types.T_BCD:
//                res = CodDatatype.BIGDECIMAL;
//                break;
//            case Types.T_DATE:
//                res = CodDatatype.DATE;
//                break;
//            case Types.T_DOM:
//                res = CodDatatype.DOM;
//                break;
//            case Types.T_BOOLEAN:
//                res = CodDatatype.BOOL;
//                break;
//            default:
//                break;
//            }
//            return res;
//        }
        public static CodDatatype getCodDatatypeByNativeName(String attrTypename) {
            for (CodDatatype cod : CodDatatype.values()) {
                if (cod.nativeName.equals(attrTypename)){
                    return cod;
                }
            }
            throw new IllegalArgumentException("Illegal typename: " + attrTypename);            
        }
    };

    public enum FilterOperator {
        lt("<"), gt(">"), ge(">="), le("<="), eq("="), neq("!="), like("WIE"), notlike("NICHT WIE");
        String stringVal;

        public static FilterOperator getFilterOperatorByString(String op) {
            for (FilterOperator fo : FilterOperator.values()) {
                if (fo.getStringValue().equalsIgnoreCase(op)) {
                    return fo;
                }
            }            
            throw new IllegalArgumentException(op);
        }

        /**
         *
         * @param op
         *            the operator by its normalized name: like, notlike, gt,
         *            ge, lt
         * @return
         */
        public static FilterOperator getFilterOperatorByNormalizedName(String op) {
            for (FilterOperator fo : FilterOperator.values()) {
                if (fo.name().equalsIgnoreCase(op)) {
                    return fo;
                }
            }
            throw new IllegalArgumentException(op);
        }

        private FilterOperator(String stringValue) {
            this.stringVal = stringValue;
        }

        public String getStringValue() {
            return stringVal;
        }

    }

    public Object getValue(CodDatatype type, String formatpattern, String txtValue) {
        Object val = txtValue;
        if (type == null) {
            return txtValue;
        }
        switch (type) {
        case STRING:
            val = txtValue;
            break;
        case DATE:
            try {
                val = new SimpleDateFormat("yyyyMMdd").parse(txtValue);
            } catch (ParseException e) {
                throw new IllegalArgumentException("failed to parse date: " + txtValue);
            }
            break;
        case BIGDECIMAL:
            val = new BigDecimal(txtValue);
            break;
        case INTEGER:
            val = new Integer(txtValue);
            break;
        case LONG:
            val = new Long(txtValue);
            break;
        case DOUBLE:
            val = new Double(txtValue);
            break;
        case FLOAT:
            val = new Float(txtValue);
            break;       
        case TIMESTAMP:            
            break;       
        case DOM:           
            break;
        default:
            break;
        }
        return val;
    }

}
