package org.openxma.addons.ui.table.customizer.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class CommonTableModel extends CommonDataModel {
    private static final long serialVersionUID = 1L;




    private List<CommonColumnModel> columnModels = new ArrayList<CommonColumnModel>();
    private String namTable;
    private String appCode;

    public CommonTableModel() {
        super();
    }


    public void setNamTable(String codTable) {
        this.namTable = codTable;
    }
    public String getNamTable() {
        return namTable;
    }
    public void setColumnModels(List<CommonColumnModel> columnModels) {
        this.columnModels = columnModels;
    }
    public void addColumnModel(CommonColumnModel columnModel) {
        this.columnModels.add(columnModel);
    }
    /*
     * return the columnModels ordered by numOrder
     */
    public List<CommonColumnModel> getColumnModels() {
         Collections.sort(columnModels,new NumOrderComp());;
         return columnModels;
    }

    public CommonColumnModel getColumnModelByColIndex(int index) {
        for (CommonColumnModel cm : columnModels) {
            if (cm.getNumColumnIndex().intValue() == index) {
                return cm;
            }
        }
        return null;
    }

    public CommonColumnModel getColumnModelByNumOrder(int numOrder) {
        for (CommonColumnModel cm : columnModels) {
            if (cm.getNumOrder().intValue() == numOrder) {
                return cm;
            }
        }
        return null;
    }
    public CommonColumnModel getColumnModelByTxtColumn (String txt) {
        for (CommonColumnModel cm : columnModels) {
            if (cm.getTxtColumn().equals(txt)) {
                return cm;
            }
        }
        return null;
    }
    public CommonColumnModel getColumnModelByName(String txt) {
        for (CommonColumnModel cm : columnModels) {
            if (cm.getNamColumn().equals(txt)) {
                return cm;
            }
        }
        return null;
    }

    public List<String> getVisibleTxtColumnsSortedByNumOrder() {
        List<String> visibles = new ArrayList<String>();
        for (CommonColumnModel cm : getColumnModels()) {
            if (cm.isVisible()) {
                visibles.add(cm.getTxtColumn());
            }
        }
        return visibles;

    }

    public List<String> getAvailableTxtColumnsAlphabetically() {
        List<String> availables = new ArrayList<String>();
        for (CommonColumnModel cm : getColumnModels()) {
            if (!cm.isVisible()) {
                availables.add(cm.getTxtColumn());
            }
        }
        Collections.sort(availables);
        return availables;

    }


    public List<CommonColumnModel> getSortableColumnsSortedByNumSort() {
        List<CommonColumnModel> sortables = new ArrayList<CommonColumnModel>();
        Collections.sort(columnModels,new NumSortComp());
        for (CommonColumnModel cm : columnModels) {
            if (cm.isToBeSorted()) {
                sortables.add(cm);
            }
        }
        return sortables;

    }
    public List<CommonColumnModel> getNotSortableColumnsSortedByNumOrder() {
        List<CommonColumnModel> nonSortables = new ArrayList<CommonColumnModel>();
        Collections.sort(columnModels,new NumOrderComp());
        for (CommonColumnModel cm : columnModels) {
            if (!cm.isToBeSorted()) {
                nonSortables.add(cm);
            }
        }
        return nonSortables;

    }

    public List<CommonColumnModel> getColumnModelsSortedByNumSort(){
        Collections.sort(getColumnModels(), new NumSortComp());
        return columnModels;
    }


    public int[] getColumnOrder() {
        int[] order = new int[columnModels.size()];
       Collections.sort(columnModels, new NumOrderComp());
       for (int i = 0; i < columnModels.size(); i++) {
           order[i] = columnModels.get(i).getNumColumnIndex();

        }
       return order;
    }

    /**
     * changes the numOrder value of the columnModel
     * where {@link CommonColumnModel#getTxtColumn()} equals txt
     * and adjustes the list
     */
    public void moveUp(String txt) {
        CommonColumnModel columnModel = getColumnModelByTxtColumn(txt);
        if (columnModel.getNumOrder()==getMinNumOrder()) { // already the top
            return;
        }
        // fetch the leading
        int lowerNumOrder = getLowerNumOrder(columnModel);
        if (lowerNumOrder == -1) {
            return;
        } else {
            CommonColumnModel columnModelByNumOrder = getColumnModelByNumOrder(lowerNumOrder);
            //switch the numOrder
            Integer tmpNumOrder = columnModelByNumOrder.getNumOrder();
            columnModelByNumOrder.setNumOrder(columnModel.getNumOrder());
            columnModel.setNumOrder(tmpNumOrder);
        }
    }
    /**
     * changes the numOrder value of the columnModel
     * where {@link CommonColumnModel#getTxtColumn()} equals txt
     * and adjustes the list
     */
    public void moveDown(String txt) {
        CommonColumnModel columnModel = getColumnModelByTxtColumn(txt);
        if (columnModel.getNumOrder()==getMaxNumOrder()) { // already the bottom
            return;
        }
        // fetch the previous
        int higherNumOrder = getHigherNumOrder(columnModel);
        if (higherNumOrder == -1){
            return;
        } else {
            CommonColumnModel columnModelByNumOrder = getColumnModelByNumOrder(higherNumOrder);
            //switch the numOrder
            Integer tmpNumOrder = columnModelByNumOrder.getNumOrder();
            columnModelByNumOrder.setNumOrder(columnModel.getNumOrder());
            columnModel.setNumOrder(tmpNumOrder);
        }
    }

    public void hide(String txt) {
        CommonColumnModel columnModel = getColumnModelByTxtColumn(txt);
        columnModel.setVisible(Boolean.FALSE);
    }
    public void show(String txt) {
        CommonColumnModel columnModel = getColumnModelByTxtColumn(txt);
        columnModel.setVisible(Boolean.TRUE);
    }

    private int getMinSortOrder(){
        Collections.sort(columnModels,new NumSortComp());
        return columnModels.get(0).getSortOrder();

    }
    private int getMinNumOrder(){
        Collections.sort(columnModels,new NumOrderComp());
        return columnModels.get(0).getNumOrder();

    }
    private int getMaxNumOrder(){
        Collections.sort(columnModels,new NumOrderComp());
        return columnModels.get(columnModels.size()-1).getNumOrder();

    }

    private int getLowerNumOrder(CommonColumnModel cm) {
        CommonColumnModel[] ca = (CommonColumnModel[]) getColumnModels().toArray(new CommonColumnModel[columnModels.size()]);
        for (int i  = 0 ; i < ca.length; i++) {
            if (cm.getNumOrder() == ca[i].getNumOrder()) {
                if (i>0){
                    return ca[i-1].getNumOrder();
                } else {
                    return -1; // already the lowest
                }
            }
        }
        return -1;
    }
    private int getHigherNumOrder(CommonColumnModel cm) {
        CommonColumnModel[] ca = (CommonColumnModel[]) getColumnModels().toArray(new CommonColumnModel[columnModels.size()]);
        for (int i  = 0 ; i < ca.length; i++) {
            if (cm.getNumOrder() == ca[i].getNumOrder()) {
                if (i<ca.length-1){
                    return ca[i+1].getNumOrder();
                } else {
                    return -1; // already the highest
                }
            }
        }
        return -1;
    }

    // decrease the numSort value
    public void sortUp(String txt) {
        CommonColumnModel columnModel = getColumnModelByName(txt);
        CommonColumnModel prevCol = getColumnModelBySortOrder(columnModel.getSortOrder()-1);
        if (prevCol == null) {
           columnModel.setSortOrder(columnModel.getSortOrder()-1);
        } else {
            Integer tmpSortOrder = prevCol.getSortOrder();
            prevCol.setSortOrder(columnModel.getSortOrder());
            columnModel.setSortOrder(tmpSortOrder);
        }
    }
    //increase the sortOrder value
    public void sortDown(String txt) {
        CommonColumnModel columnModel = getColumnModelByName(txt);
        if (columnModel.getSortOrder() == Integer.MAX_VALUE) {
            //already down
            return;
        }
        // fetch the lower sort val
        CommonColumnModel lowerCol = getColumnModelBySortOrder(columnModel.getSortOrder() + 1);
        if (lowerCol == null) { // no col with lower sort val
            columnModel.setSortOrder(columnModel.getSortOrder()+1);
        } else {
            Integer tmpSortOrder = lowerCol.getSortOrder();
            lowerCol.setSortOrder(columnModel.getSortOrder());
            columnModel.setSortOrder(tmpSortOrder);
        }
    }

    private CommonColumnModel getColumnModelBySortOrder(int sortOrder) {
        for (CommonColumnModel cm : columnModels) {
            if (cm.getSortOrder().intValue() == sortOrder) {
                return cm;
            }
        }
        return null;

    }

    @Override
    public boolean equals(Object obj) {
        if (this.namTable == null) {
            return false;
        }
        return this.namTable.equals(((CommonTableModel) obj).namTable);
    }

    @Override
    public int hashCode() {
        if (this.namTable == null) {
            return super.hashCode();
        } else {
            return this.namTable.hashCode();
        }
    }
    /**
     *
     * sets the given column as sortable
     * by setting the sorting direction to ascending<br>
     * and setting the sortOrder to the biggest sortorder + 1
     *
     *
     */
    public void setSortable(CommonColumnModel columnModelByName) {
       List<CommonColumnModel> sortables = getSortableColumnsSortedByNumSort();
       columnModelByName.setAscending(Boolean.TRUE);
       if (sortables.size()==0) {
           columnModelByName.setSortOrder(0);
       } else {
           columnModelByName.setSortOrder(sortables.get(sortables.size()-1).getSortOrder()+1);
       }

    }

    public void setAppCode(String appCode) {
        this.appCode = appCode;
    }
    public String getAppCode() {
        return appCode;
    }

    private class NumOrderComp implements Comparator<CommonColumnModel>, Serializable {
        /**
         *
         */
        private static final long serialVersionUID = 1;

        public int compare(CommonColumnModel c1, CommonColumnModel c2) {
            if (c1.getNumOrder() < c2.getNumOrder()) {
                return -1;
            } else if (c1.getNumOrder() == c2.getNumOrder()) {
                return 0;
            } else {
                return 1;
            }
        }
    }

    private class NumSortComp implements Comparator<CommonColumnModel>, Serializable {
        /**
         *
         */
        private static final long serialVersionUID = 1;

        public int compare(CommonColumnModel c1, CommonColumnModel c2) {
            if (c1.getSortOrder() == Integer.MAX_VALUE && c2.getSortOrder() == Integer.MAX_VALUE) {
                if (c1.getNumOrder() < c2.getNumOrder()) {
                    return -1;
                } else if (c1.getNumOrder() > c2.getNumOrder()) {
                    return 1;
                } else {
                    return 0;
                }
            } else if (c1.getSortOrder() < c2.getSortOrder()) {
                return -1;
            } else if (c1.getSortOrder() == c2.getSortOrder()) {
                return 0;
            } else {
                return 1;
            }
        }
    }
    
    /**
     * removes all filters for all columns for this tableModel.
     * 
     * @since 4.0.0
     * @author s4162
     */
    public void removeAllFilter(){
        for (CommonColumnModel cm : getColumnModels()){
            cm.removeFilterModel();
        }
    }



}
