package org.openxma.addons.ui.table.customizer.mdl;

import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerColumnView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerEntityView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerFilterView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerTableView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerUserView;
import org.openxma.addons.ui.table.customizer.mdl.dto.UserTableView;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerColumn;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerEntity;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerFilter;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerTable;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerUser;
import org.openxma.dsl.platform.assembler.DtoAssembler;
import org.openxma.dsl.platform.assembler.DtoAssemblerFactory;
import org.openxma.dsl.platform.assembler.DtoAssemblerRegistry;
import org.openxma.dsl.platform.converter.Converter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;

public class DtoAssemblerFactoryMdlGen implements DtoAssemblerFactory {

	private List<DtoAssembler<?>> dtoAssemblers;

	@Autowired
    private DtoAssemblerRegistry dtoAssemblerRegistry;
    
    @Autowired
    private Converter converter;

    public DtoAssemblerFactoryMdlGen() {
		dtoAssemblers = new ArrayList<DtoAssembler<?>>(6);
		initDtoAssemblers();
	}


	public List<DtoAssembler<?>> getDtoAssemblers() {
		return dtoAssemblers;
	}

	protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
		return dtoAssemblerRegistry;
	}
	
	protected Converter getConverter() {
		return converter;
	}

	private void initDtoAssemblers() {
		dtoAssemblers.add(createDtoAssemblerTableCustomizerColumnView());
		dtoAssemblers.add(createDtoAssemblerTableCustomizerEntityView());
		dtoAssemblers.add(createDtoAssemblerTableCustomizerFilterView());
		dtoAssemblers.add(createDtoAssemblerTableCustomizerTableView());
		dtoAssemblers.add(createDtoAssemblerTableCustomizerUserView());
		dtoAssemblers.add(createDtoAssemblerUserTableView());
	}


	private DtoAssembler<TableCustomizerColumnView> createDtoAssemblerTableCustomizerColumnView() {
		return new DtoAssembler<TableCustomizerColumnView>() {
		    public TableCustomizerColumnView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerColumn) {
					TableCustomizerColumn tableCustomizerColumn = (TableCustomizerColumn) sourceEntity;
		    		TableCustomizerColumnView result = mapFromTableCustomizerColumnToTableCustomizerColumnView(tableCustomizerColumn);
		    		if (null != tableCustomizerColumn.getTableCustomizerTable()) {
		    			result.setTableCustomizerTableOid(tableCustomizerColumn.getTableCustomizerTable().getOid());
		    		}
		    		return result;
		    	} else if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		TableCustomizerColumnView result = mapFromTableCustomizerTableToTableCustomizerColumnView(tableCustomizerTable);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public TableCustomizerColumnView mapFromEntity(Object sourceEntity, TableCustomizerColumnView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerColumn) {
					TableCustomizerColumn tableCustomizerColumn = (TableCustomizerColumn) sourceEntity;
		    		mapFromTableCustomizerColumnToTableCustomizerColumnView(tableCustomizerColumn, targetDto);
		    		if (null != tableCustomizerColumn.getTableCustomizerTable()) {
		    			targetDto.setTableCustomizerTableOid(tableCustomizerColumn.getTableCustomizerTable().getOid());
		    		}
		    		return targetDto;
		    	} else if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		mapFromTableCustomizerTableToTableCustomizerColumnView(tableCustomizerTable, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(TableCustomizerColumnView sourceDto, Class<T> target) {
				if (TableCustomizerColumnView.class.equals(target)) {
					TableCustomizerColumnView result = new TableCustomizerColumnView();
					mapFromTableCustomizerColumnViewToTableCustomizerColumnView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerColumn.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerColumnViewToTableCustomizerColumn(sourceDto));
		    	}
				else if (TableCustomizerTable.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerColumnViewToTableCustomizerTable(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(TableCustomizerColumnView sourceDto, Object targetObject) {
		    	if (targetObject instanceof TableCustomizerColumnView) {
					mapFromTableCustomizerColumnViewToTableCustomizerColumnView(sourceDto, (TableCustomizerColumnView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerColumn) {
		    		mapFromTableCustomizerColumnViewToTableCustomizerColumn(sourceDto, (TableCustomizerColumn)targetObject);
		    	}		    	
		    	else if (targetObject instanceof TableCustomizerTable) {
		    		mapFromTableCustomizerColumnViewToTableCustomizerTable(sourceDto, (TableCustomizerTable)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromTableCustomizerColumnViewToTableCustomizerColumnView(TableCustomizerColumnView sourceDto, TableCustomizerColumnView tableCustomizerColumnView) {
   		tableCustomizerColumnView.setTableCustomizerFilter(sourceDto.getTableCustomizerFilter());
   		tableCustomizerColumnView.setOid(sourceDto.getOid());
   		tableCustomizerColumnView.setCodOwner(sourceDto.getCodOwner());
   		tableCustomizerColumnView.setNumMandant(sourceDto.getNumMandant());
   		tableCustomizerColumnView.setFipUser(sourceDto.getFipUser());
   		tableCustomizerColumnView.setFipDate(sourceDto.getFipDate());
   		tableCustomizerColumnView.setNamColumn(sourceDto.getNamColumn());
   		tableCustomizerColumnView.setTxtColumn(sourceDto.getTxtColumn());
   		tableCustomizerColumnView.setNumSort(sourceDto.getNumSort());
   		tableCustomizerColumnView.setBolAscending(sourceDto.getBolAscending());
   		tableCustomizerColumnView.setNumOrder(sourceDto.getNumOrder());
   		tableCustomizerColumnView.setBolVisible(sourceDto.getBolVisible());
   		tableCustomizerColumnView.setNumDefaultWidth(sourceDto.getNumDefaultWidth());
   		tableCustomizerColumnView.setNumColumnIndex(sourceDto.getNumColumnIndex());
   		tableCustomizerColumnView.setRatDefault(sourceDto.getRatDefault());
   		tableCustomizerColumnView.setBolIsStretchColumn(sourceDto.getBolIsStretchColumn());
   		tableCustomizerColumnView.setNumDefaultMinWith(sourceDto.getNumDefaultMinWith());
   		tableCustomizerColumnView.setBolIsResizable(sourceDto.getBolIsResizable());
   		tableCustomizerColumnView.setCodDataType(sourceDto.getCodDataType());
   		tableCustomizerColumnView.setBolAndFilter(sourceDto.getBolAndFilter());
   		tableCustomizerColumnView.setBolIsNullableFilter(sourceDto.getBolIsNullableFilter());
   		tableCustomizerColumnView.setTxtFormatterPattern(sourceDto.getTxtFormatterPattern());
   		tableCustomizerColumnView.setTableCustomizerTableOid(sourceDto.getTableCustomizerTableOid());
    }
    
    protected TableCustomizerColumn mapFromTableCustomizerColumnViewToTableCustomizerColumn(TableCustomizerColumnView sourceDto) {
    	return mapFromTableCustomizerColumnViewToTableCustomizerColumn(sourceDto, MapperContextHolder.createEntity(TableCustomizerColumn.class));
    }

    protected TableCustomizerColumn mapFromTableCustomizerColumnViewToTableCustomizerColumn(TableCustomizerColumnView sourceDto, TableCustomizerColumn tableCustomizerColumn) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerColumn;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerColumn);
		if (sourceDto.getOid() != null) {
			tableCustomizerColumn.setOid(sourceDto.getOid());
		}
		tableCustomizerColumn.setCodOwner(sourceDto.getCodOwner());
		tableCustomizerColumn.setNumMandant(sourceDto.getNumMandant());
		tableCustomizerColumn.setFipUser(sourceDto.getFipUser());
		tableCustomizerColumn.setFipDate(sourceDto.getFipDate());
		tableCustomizerColumn.setNamColumn(sourceDto.getNamColumn());
		tableCustomizerColumn.setTxtColumn(sourceDto.getTxtColumn());
		tableCustomizerColumn.setNumSort(sourceDto.getNumSort());
		tableCustomizerColumn.setBolAscending(sourceDto.getBolAscending());
		tableCustomizerColumn.setNumOrder(sourceDto.getNumOrder());
		tableCustomizerColumn.setBolVisible(sourceDto.getBolVisible());
		tableCustomizerColumn.setNumDefaultWidth(sourceDto.getNumDefaultWidth());
		tableCustomizerColumn.setNumColumnIndex(sourceDto.getNumColumnIndex());
		tableCustomizerColumn.setRatDefault(sourceDto.getRatDefault());
		tableCustomizerColumn.setBolIsStretchColumn(sourceDto.getBolIsStretchColumn());
		tableCustomizerColumn.setNumDefaultMinWith(sourceDto.getNumDefaultMinWith());
		tableCustomizerColumn.setBolIsResizable(sourceDto.getBolIsResizable());
		tableCustomizerColumn.setCodDataType(sourceDto.getCodDataType());
		tableCustomizerColumn.setBolAndFilter(sourceDto.getBolAndFilter());
		tableCustomizerColumn.setBolIsNullableFilter(sourceDto.getBolIsNullableFilter());
		tableCustomizerColumn.setTxtFormatterPattern(sourceDto.getTxtFormatterPattern());
	    if (!MapperContextHolder.isContext("saveTableCustomizerColumn") && !MapperContextHolder.isContext("updateTableCustomizerColumn")) {
			if (null != sourceDto.getTableCustomizerFilter()) {
				for (TableCustomizerFilterView tableCustomizerFilterView : sourceDto.getTableCustomizerFilter()) {
					TableCustomizerFilter tableCustomizerFilter = (TableCustomizerFilter) dtoAssemblerRegistry.getDtoAssembler(TableCustomizerFilterView.class).mapTo(tableCustomizerFilterView, TableCustomizerFilter.class);
					tableCustomizerColumn.addTableCustomizerFilter(tableCustomizerFilter);
				}
			}
	    }
	    return tableCustomizerColumn;
	}
			
    protected TableCustomizerTable mapFromTableCustomizerColumnViewToTableCustomizerTable(TableCustomizerColumnView sourceDto) {
    	return mapFromTableCustomizerColumnViewToTableCustomizerTable(sourceDto, MapperContextHolder.createEntity(TableCustomizerTable.class));
    }

    protected TableCustomizerTable mapFromTableCustomizerColumnViewToTableCustomizerTable(TableCustomizerColumnView sourceDto, TableCustomizerTable tableCustomizerTable) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerTable;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerTable);
		if (sourceDto.getTableCustomizerTableOid() != null  && MapperContextHolder.isContext("tableCustomizerTable")) {
			tableCustomizerTable.setOid(sourceDto.getTableCustomizerTableOid());
		}
	    return tableCustomizerTable;
	}
	
	protected TableCustomizerColumnView mapFromTableCustomizerColumnToTableCustomizerColumnView(TableCustomizerColumn tableCustomizerColumn) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerColumn)) {
			return (TableCustomizerColumnView) MapperContextHolder.getContext().get(tableCustomizerColumn);
		}
		TableCustomizerColumnView tableCustomizerColumnView = new TableCustomizerColumnView();
    	MapperContextHolder.getContext().put(tableCustomizerColumn, tableCustomizerColumnView);
		tableCustomizerColumnView.setOid(tableCustomizerColumn.getOid());
		tableCustomizerColumnView.setCodOwner(tableCustomizerColumn.getCodOwner());
		tableCustomizerColumnView.setNumMandant(tableCustomizerColumn.getNumMandant());
		tableCustomizerColumnView.setFipUser(tableCustomizerColumn.getFipUser());
		tableCustomizerColumnView.setFipDate(tableCustomizerColumn.getFipDate());
		tableCustomizerColumnView.setNamColumn(tableCustomizerColumn.getNamColumn());
		tableCustomizerColumnView.setTxtColumn(tableCustomizerColumn.getTxtColumn());
		tableCustomizerColumnView.setNumSort(tableCustomizerColumn.getNumSort());
		tableCustomizerColumnView.setBolAscending(tableCustomizerColumn.getBolAscending());
		tableCustomizerColumnView.setNumOrder(tableCustomizerColumn.getNumOrder());
		tableCustomizerColumnView.setBolVisible(tableCustomizerColumn.getBolVisible());
		tableCustomizerColumnView.setNumDefaultWidth(tableCustomizerColumn.getNumDefaultWidth());
		tableCustomizerColumnView.setNumColumnIndex(tableCustomizerColumn.getNumColumnIndex());
		tableCustomizerColumnView.setRatDefault(tableCustomizerColumn.getRatDefault());
		tableCustomizerColumnView.setBolIsStretchColumn(tableCustomizerColumn.getBolIsStretchColumn());
		tableCustomizerColumnView.setNumDefaultMinWith(tableCustomizerColumn.getNumDefaultMinWith());
		tableCustomizerColumnView.setBolIsResizable(tableCustomizerColumn.getBolIsResizable());
		tableCustomizerColumnView.setCodDataType(tableCustomizerColumn.getCodDataType());
		tableCustomizerColumnView.setBolAndFilter(tableCustomizerColumn.getBolAndFilter());
		tableCustomizerColumnView.setBolIsNullableFilter(tableCustomizerColumn.getBolIsNullableFilter());
		tableCustomizerColumnView.setTxtFormatterPattern(tableCustomizerColumn.getTxtFormatterPattern());
		if (tableCustomizerColumn.hasTableCustomizerFilter()) {
			for (TableCustomizerFilter next : tableCustomizerColumn.getTableCustomizerFilter()) {
	    		TableCustomizerFilterView nextTableCustomizerFilterView = (TableCustomizerFilterView) MapperContextHolder.getContext().get(next);
	    		if (null == nextTableCustomizerFilterView) {
		        	nextTableCustomizerFilterView = dtoAssemblerRegistry.getDtoAssembler(TableCustomizerFilterView.class).mapFromEntity(next);
		        }
	        	tableCustomizerColumnView.addTableCustomizerFilter(nextTableCustomizerFilterView);
	    	}
		}
        return tableCustomizerColumnView;
	}
	
	protected TableCustomizerColumnView mapFromTableCustomizerColumnToTableCustomizerColumnView(TableCustomizerColumn tableCustomizerColumn, TableCustomizerColumnView tableCustomizerColumnView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerColumn)) {
			return tableCustomizerColumnView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerColumn, tableCustomizerColumnView);
		tableCustomizerColumnView.setOid(tableCustomizerColumn.getOid());
		tableCustomizerColumnView.setCodOwner(tableCustomizerColumn.getCodOwner());
		tableCustomizerColumnView.setNumMandant(tableCustomizerColumn.getNumMandant());
		tableCustomizerColumnView.setFipUser(tableCustomizerColumn.getFipUser());
		tableCustomizerColumnView.setFipDate(tableCustomizerColumn.getFipDate());
		tableCustomizerColumnView.setNamColumn(tableCustomizerColumn.getNamColumn());
		tableCustomizerColumnView.setTxtColumn(tableCustomizerColumn.getTxtColumn());
		tableCustomizerColumnView.setNumSort(tableCustomizerColumn.getNumSort());
		tableCustomizerColumnView.setBolAscending(tableCustomizerColumn.getBolAscending());
		tableCustomizerColumnView.setNumOrder(tableCustomizerColumn.getNumOrder());
		tableCustomizerColumnView.setBolVisible(tableCustomizerColumn.getBolVisible());
		tableCustomizerColumnView.setNumDefaultWidth(tableCustomizerColumn.getNumDefaultWidth());
		tableCustomizerColumnView.setNumColumnIndex(tableCustomizerColumn.getNumColumnIndex());
		tableCustomizerColumnView.setRatDefault(tableCustomizerColumn.getRatDefault());
		tableCustomizerColumnView.setBolIsStretchColumn(tableCustomizerColumn.getBolIsStretchColumn());
		tableCustomizerColumnView.setNumDefaultMinWith(tableCustomizerColumn.getNumDefaultMinWith());
		tableCustomizerColumnView.setBolIsResizable(tableCustomizerColumn.getBolIsResizable());
		tableCustomizerColumnView.setCodDataType(tableCustomizerColumn.getCodDataType());
		tableCustomizerColumnView.setBolAndFilter(tableCustomizerColumn.getBolAndFilter());
		tableCustomizerColumnView.setBolIsNullableFilter(tableCustomizerColumn.getBolIsNullableFilter());
		tableCustomizerColumnView.setTxtFormatterPattern(tableCustomizerColumn.getTxtFormatterPattern());
		if (tableCustomizerColumn.hasTableCustomizerFilter()) {
			for (TableCustomizerFilter next : tableCustomizerColumn.getTableCustomizerFilter()) {
	    		TableCustomizerFilterView nextTableCustomizerFilterView = (TableCustomizerFilterView) MapperContextHolder.getContext().get(next);
	    		if (null == nextTableCustomizerFilterView) {
		        	nextTableCustomizerFilterView = dtoAssemblerRegistry.getDtoAssembler(TableCustomizerFilterView.class).mapFromEntity(next);
		        }
	        	tableCustomizerColumnView.addTableCustomizerFilter(nextTableCustomizerFilterView);
	    	}
		}
        return tableCustomizerColumnView;
	}
	
	protected TableCustomizerColumnView mapFromTableCustomizerTableToTableCustomizerColumnView(TableCustomizerTable tableCustomizerTable) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return (TableCustomizerColumnView) MapperContextHolder.getContext().get(tableCustomizerTable);
		}
		TableCustomizerColumnView tableCustomizerColumnView = new TableCustomizerColumnView();
    	MapperContextHolder.getContext().put(tableCustomizerTable, tableCustomizerColumnView);
		tableCustomizerColumnView.setTableCustomizerTableOid(tableCustomizerTable.getOid());
        return tableCustomizerColumnView;
	}
	
	protected TableCustomizerColumnView mapFromTableCustomizerTableToTableCustomizerColumnView(TableCustomizerTable tableCustomizerTable, TableCustomizerColumnView tableCustomizerColumnView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return tableCustomizerColumnView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerTable, tableCustomizerColumnView);
		tableCustomizerColumnView.setTableCustomizerTableOid(tableCustomizerTable.getOid());
        return tableCustomizerColumnView;
	}
	
	

	private DtoAssembler<TableCustomizerEntityView> createDtoAssemblerTableCustomizerEntityView() {
		return new DtoAssembler<TableCustomizerEntityView>() {
		    public TableCustomizerEntityView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerEntity) {
					TableCustomizerEntity tableCustomizerEntity = (TableCustomizerEntity) sourceEntity;
		    		TableCustomizerEntityView result = mapFromTableCustomizerEntityToTableCustomizerEntityView(tableCustomizerEntity);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public TableCustomizerEntityView mapFromEntity(Object sourceEntity, TableCustomizerEntityView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerEntity) {
					TableCustomizerEntity tableCustomizerEntity = (TableCustomizerEntity) sourceEntity;
		    		mapFromTableCustomizerEntityToTableCustomizerEntityView(tableCustomizerEntity, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(TableCustomizerEntityView sourceDto, Class<T> target) {
				if (TableCustomizerEntityView.class.equals(target)) {
					TableCustomizerEntityView result = new TableCustomizerEntityView();
					mapFromTableCustomizerEntityViewToTableCustomizerEntityView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerEntity.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerEntityViewToTableCustomizerEntity(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(TableCustomizerEntityView sourceDto, Object targetObject) {
		    	if (targetObject instanceof TableCustomizerEntityView) {
					mapFromTableCustomizerEntityViewToTableCustomizerEntityView(sourceDto, (TableCustomizerEntityView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerEntity) {
		    		mapFromTableCustomizerEntityViewToTableCustomizerEntity(sourceDto, (TableCustomizerEntity)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromTableCustomizerEntityViewToTableCustomizerEntityView(TableCustomizerEntityView sourceDto, TableCustomizerEntityView tableCustomizerEntityView) {
   		tableCustomizerEntityView.setOid(sourceDto.getOid());
   		tableCustomizerEntityView.setCodOwner(sourceDto.getCodOwner());
   		tableCustomizerEntityView.setNumMandant(sourceDto.getNumMandant());
   		tableCustomizerEntityView.setFipUser(sourceDto.getFipUser());
   		tableCustomizerEntityView.setFipDate(sourceDto.getFipDate());
    }
    
    protected TableCustomizerEntity mapFromTableCustomizerEntityViewToTableCustomizerEntity(TableCustomizerEntityView sourceDto) {
    	return mapFromTableCustomizerEntityViewToTableCustomizerEntity(sourceDto, MapperContextHolder.createEntity(TableCustomizerEntity.class));
    }

    protected TableCustomizerEntity mapFromTableCustomizerEntityViewToTableCustomizerEntity(TableCustomizerEntityView sourceDto, TableCustomizerEntity tableCustomizerEntity) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerEntity;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerEntity);
		if (sourceDto.getOid() != null) {
			tableCustomizerEntity.setOid(sourceDto.getOid());
		}
		tableCustomizerEntity.setCodOwner(sourceDto.getCodOwner());
		tableCustomizerEntity.setNumMandant(sourceDto.getNumMandant());
		tableCustomizerEntity.setFipUser(sourceDto.getFipUser());
		tableCustomizerEntity.setFipDate(sourceDto.getFipDate());
	    return tableCustomizerEntity;
	}
	
	protected TableCustomizerEntityView mapFromTableCustomizerEntityToTableCustomizerEntityView(TableCustomizerEntity tableCustomizerEntity) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerEntity)) {
			return (TableCustomizerEntityView) MapperContextHolder.getContext().get(tableCustomizerEntity);
		}
		TableCustomizerEntityView tableCustomizerEntityView = new TableCustomizerEntityView();
    	MapperContextHolder.getContext().put(tableCustomizerEntity, tableCustomizerEntityView);
		tableCustomizerEntityView.setOid(tableCustomizerEntity.getOid());
		tableCustomizerEntityView.setCodOwner(tableCustomizerEntity.getCodOwner());
		tableCustomizerEntityView.setNumMandant(tableCustomizerEntity.getNumMandant());
		tableCustomizerEntityView.setFipUser(tableCustomizerEntity.getFipUser());
		tableCustomizerEntityView.setFipDate(tableCustomizerEntity.getFipDate());
        return tableCustomizerEntityView;
	}
	
	protected TableCustomizerEntityView mapFromTableCustomizerEntityToTableCustomizerEntityView(TableCustomizerEntity tableCustomizerEntity, TableCustomizerEntityView tableCustomizerEntityView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerEntity)) {
			return tableCustomizerEntityView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerEntity, tableCustomizerEntityView);
		tableCustomizerEntityView.setOid(tableCustomizerEntity.getOid());
		tableCustomizerEntityView.setCodOwner(tableCustomizerEntity.getCodOwner());
		tableCustomizerEntityView.setNumMandant(tableCustomizerEntity.getNumMandant());
		tableCustomizerEntityView.setFipUser(tableCustomizerEntity.getFipUser());
		tableCustomizerEntityView.setFipDate(tableCustomizerEntity.getFipDate());
        return tableCustomizerEntityView;
	}
	
	

	private DtoAssembler<TableCustomizerFilterView> createDtoAssemblerTableCustomizerFilterView() {
		return new DtoAssembler<TableCustomizerFilterView>() {
		    public TableCustomizerFilterView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerFilter) {
					TableCustomizerFilter tableCustomizerFilter = (TableCustomizerFilter) sourceEntity;
		    		TableCustomizerFilterView result = mapFromTableCustomizerFilterToTableCustomizerFilterView(tableCustomizerFilter);
		    		if (null != tableCustomizerFilter.getTableCustomizerColumn()) {
		    			result.setTableCustomizerColumnOid(tableCustomizerFilter.getTableCustomizerColumn().getOid());
		    		}
		    		return result;
		    	} else if (sourceEntity instanceof TableCustomizerColumn) {
					TableCustomizerColumn tableCustomizerColumn = (TableCustomizerColumn) sourceEntity;
		    		TableCustomizerFilterView result = mapFromTableCustomizerColumnToTableCustomizerFilterView(tableCustomizerColumn);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public TableCustomizerFilterView mapFromEntity(Object sourceEntity, TableCustomizerFilterView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerFilter) {
					TableCustomizerFilter tableCustomizerFilter = (TableCustomizerFilter) sourceEntity;
		    		mapFromTableCustomizerFilterToTableCustomizerFilterView(tableCustomizerFilter, targetDto);
		    		if (null != tableCustomizerFilter.getTableCustomizerColumn()) {
		    			targetDto.setTableCustomizerColumnOid(tableCustomizerFilter.getTableCustomizerColumn().getOid());
		    		}
		    		return targetDto;
		    	} else if (sourceEntity instanceof TableCustomizerColumn) {
					TableCustomizerColumn tableCustomizerColumn = (TableCustomizerColumn) sourceEntity;
		    		mapFromTableCustomizerColumnToTableCustomizerFilterView(tableCustomizerColumn, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(TableCustomizerFilterView sourceDto, Class<T> target) {
				if (TableCustomizerFilterView.class.equals(target)) {
					TableCustomizerFilterView result = new TableCustomizerFilterView();
					mapFromTableCustomizerFilterViewToTableCustomizerFilterView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerFilter.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerFilterViewToTableCustomizerFilter(sourceDto));
		    	}
				else if (TableCustomizerColumn.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerFilterViewToTableCustomizerColumn(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(TableCustomizerFilterView sourceDto, Object targetObject) {
		    	if (targetObject instanceof TableCustomizerFilterView) {
					mapFromTableCustomizerFilterViewToTableCustomizerFilterView(sourceDto, (TableCustomizerFilterView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerFilter) {
		    		mapFromTableCustomizerFilterViewToTableCustomizerFilter(sourceDto, (TableCustomizerFilter)targetObject);
		    	}		    	
		    	else if (targetObject instanceof TableCustomizerColumn) {
		    		mapFromTableCustomizerFilterViewToTableCustomizerColumn(sourceDto, (TableCustomizerColumn)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromTableCustomizerFilterViewToTableCustomizerFilterView(TableCustomizerFilterView sourceDto, TableCustomizerFilterView tableCustomizerFilterView) {
   		tableCustomizerFilterView.setOid(sourceDto.getOid());
   		tableCustomizerFilterView.setCodOwner(sourceDto.getCodOwner());
   		tableCustomizerFilterView.setNumMandant(sourceDto.getNumMandant());
   		tableCustomizerFilterView.setFipUser(sourceDto.getFipUser());
   		tableCustomizerFilterView.setFipDate(sourceDto.getFipDate());
   		tableCustomizerFilterView.setTxtValue(sourceDto.getTxtValue());
   		tableCustomizerFilterView.setCodFilterOperator(sourceDto.getCodFilterOperator());
   		tableCustomizerFilterView.setTableCustomizerColumnOid(sourceDto.getTableCustomizerColumnOid());
    }
    
    protected TableCustomizerFilter mapFromTableCustomizerFilterViewToTableCustomizerFilter(TableCustomizerFilterView sourceDto) {
    	return mapFromTableCustomizerFilterViewToTableCustomizerFilter(sourceDto, MapperContextHolder.createEntity(TableCustomizerFilter.class));
    }

    protected TableCustomizerFilter mapFromTableCustomizerFilterViewToTableCustomizerFilter(TableCustomizerFilterView sourceDto, TableCustomizerFilter tableCustomizerFilter) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerFilter;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerFilter);
		if (sourceDto.getOid() != null) {
			tableCustomizerFilter.setOid(sourceDto.getOid());
		}
		tableCustomizerFilter.setCodOwner(sourceDto.getCodOwner());
		tableCustomizerFilter.setNumMandant(sourceDto.getNumMandant());
		tableCustomizerFilter.setFipUser(sourceDto.getFipUser());
		tableCustomizerFilter.setFipDate(sourceDto.getFipDate());
		tableCustomizerFilter.setTxtValue(sourceDto.getTxtValue());
		tableCustomizerFilter.setCodFilterOperator(sourceDto.getCodFilterOperator());
	    return tableCustomizerFilter;
	}
			
    protected TableCustomizerColumn mapFromTableCustomizerFilterViewToTableCustomizerColumn(TableCustomizerFilterView sourceDto) {
    	return mapFromTableCustomizerFilterViewToTableCustomizerColumn(sourceDto, MapperContextHolder.createEntity(TableCustomizerColumn.class));
    }

    protected TableCustomizerColumn mapFromTableCustomizerFilterViewToTableCustomizerColumn(TableCustomizerFilterView sourceDto, TableCustomizerColumn tableCustomizerColumn) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerColumn;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerColumn);
		if (sourceDto.getTableCustomizerColumnOid() != null  && MapperContextHolder.isContext("tableCustomizerColumn")) {
			tableCustomizerColumn.setOid(sourceDto.getTableCustomizerColumnOid());
		}
	    return tableCustomizerColumn;
	}
	
	protected TableCustomizerFilterView mapFromTableCustomizerFilterToTableCustomizerFilterView(TableCustomizerFilter tableCustomizerFilter) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerFilter)) {
			return (TableCustomizerFilterView) MapperContextHolder.getContext().get(tableCustomizerFilter);
		}
		TableCustomizerFilterView tableCustomizerFilterView = new TableCustomizerFilterView();
    	MapperContextHolder.getContext().put(tableCustomizerFilter, tableCustomizerFilterView);
		tableCustomizerFilterView.setOid(tableCustomizerFilter.getOid());
		tableCustomizerFilterView.setCodOwner(tableCustomizerFilter.getCodOwner());
		tableCustomizerFilterView.setNumMandant(tableCustomizerFilter.getNumMandant());
		tableCustomizerFilterView.setFipUser(tableCustomizerFilter.getFipUser());
		tableCustomizerFilterView.setFipDate(tableCustomizerFilter.getFipDate());
		tableCustomizerFilterView.setTxtValue(tableCustomizerFilter.getTxtValue());
		tableCustomizerFilterView.setCodFilterOperator(tableCustomizerFilter.getCodFilterOperator());
        return tableCustomizerFilterView;
	}
	
	protected TableCustomizerFilterView mapFromTableCustomizerFilterToTableCustomizerFilterView(TableCustomizerFilter tableCustomizerFilter, TableCustomizerFilterView tableCustomizerFilterView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerFilter)) {
			return tableCustomizerFilterView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerFilter, tableCustomizerFilterView);
		tableCustomizerFilterView.setOid(tableCustomizerFilter.getOid());
		tableCustomizerFilterView.setCodOwner(tableCustomizerFilter.getCodOwner());
		tableCustomizerFilterView.setNumMandant(tableCustomizerFilter.getNumMandant());
		tableCustomizerFilterView.setFipUser(tableCustomizerFilter.getFipUser());
		tableCustomizerFilterView.setFipDate(tableCustomizerFilter.getFipDate());
		tableCustomizerFilterView.setTxtValue(tableCustomizerFilter.getTxtValue());
		tableCustomizerFilterView.setCodFilterOperator(tableCustomizerFilter.getCodFilterOperator());
        return tableCustomizerFilterView;
	}
	
	protected TableCustomizerFilterView mapFromTableCustomizerColumnToTableCustomizerFilterView(TableCustomizerColumn tableCustomizerColumn) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerColumn)) {
			return (TableCustomizerFilterView) MapperContextHolder.getContext().get(tableCustomizerColumn);
		}
		TableCustomizerFilterView tableCustomizerFilterView = new TableCustomizerFilterView();
    	MapperContextHolder.getContext().put(tableCustomizerColumn, tableCustomizerFilterView);
		tableCustomizerFilterView.setTableCustomizerColumnOid(tableCustomizerColumn.getOid());
        return tableCustomizerFilterView;
	}
	
	protected TableCustomizerFilterView mapFromTableCustomizerColumnToTableCustomizerFilterView(TableCustomizerColumn tableCustomizerColumn, TableCustomizerFilterView tableCustomizerFilterView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerColumn)) {
			return tableCustomizerFilterView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerColumn, tableCustomizerFilterView);
		tableCustomizerFilterView.setTableCustomizerColumnOid(tableCustomizerColumn.getOid());
        return tableCustomizerFilterView;
	}
	
	

	private DtoAssembler<TableCustomizerTableView> createDtoAssemblerTableCustomizerTableView() {
		return new DtoAssembler<TableCustomizerTableView>() {
		    public TableCustomizerTableView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		TableCustomizerTableView result = mapFromTableCustomizerTableToTableCustomizerTableView(tableCustomizerTable);
		    		if (null != tableCustomizerTable.getTableCustomizerUser()) {
		    			result.setTableCustomizerUserOid(tableCustomizerTable.getTableCustomizerUser().getOid());
		    		}
		    		return result;
		    	} else if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		TableCustomizerTableView result = mapFromTableCustomizerUserToTableCustomizerTableView(tableCustomizerUser);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public TableCustomizerTableView mapFromEntity(Object sourceEntity, TableCustomizerTableView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		mapFromTableCustomizerTableToTableCustomizerTableView(tableCustomizerTable, targetDto);
		    		if (null != tableCustomizerTable.getTableCustomizerUser()) {
		    			targetDto.setTableCustomizerUserOid(tableCustomizerTable.getTableCustomizerUser().getOid());
		    		}
		    		return targetDto;
		    	} else if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		mapFromTableCustomizerUserToTableCustomizerTableView(tableCustomizerUser, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(TableCustomizerTableView sourceDto, Class<T> target) {
				if (TableCustomizerTableView.class.equals(target)) {
					TableCustomizerTableView result = new TableCustomizerTableView();
					mapFromTableCustomizerTableViewToTableCustomizerTableView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerTable.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerTableViewToTableCustomizerTable(sourceDto));
		    	}
				else if (TableCustomizerUser.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerTableViewToTableCustomizerUser(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(TableCustomizerTableView sourceDto, Object targetObject) {
		    	if (targetObject instanceof TableCustomizerTableView) {
					mapFromTableCustomizerTableViewToTableCustomizerTableView(sourceDto, (TableCustomizerTableView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerTable) {
		    		mapFromTableCustomizerTableViewToTableCustomizerTable(sourceDto, (TableCustomizerTable)targetObject);
		    	}		    	
		    	else if (targetObject instanceof TableCustomizerUser) {
		    		mapFromTableCustomizerTableViewToTableCustomizerUser(sourceDto, (TableCustomizerUser)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromTableCustomizerTableViewToTableCustomizerTableView(TableCustomizerTableView sourceDto, TableCustomizerTableView tableCustomizerTableView) {
   		tableCustomizerTableView.setOid(sourceDto.getOid());
   		tableCustomizerTableView.setCodOwner(sourceDto.getCodOwner());
   		tableCustomizerTableView.setNumMandant(sourceDto.getNumMandant());
   		tableCustomizerTableView.setFipUser(sourceDto.getFipUser());
   		tableCustomizerTableView.setFipDate(sourceDto.getFipDate());
   		tableCustomizerTableView.setNamTable(sourceDto.getNamTable());
   		tableCustomizerTableView.setTableCustomizerUserOid(sourceDto.getTableCustomizerUserOid());
    }
    
    protected TableCustomizerTable mapFromTableCustomizerTableViewToTableCustomizerTable(TableCustomizerTableView sourceDto) {
    	return mapFromTableCustomizerTableViewToTableCustomizerTable(sourceDto, MapperContextHolder.createEntity(TableCustomizerTable.class));
    }

    protected TableCustomizerTable mapFromTableCustomizerTableViewToTableCustomizerTable(TableCustomizerTableView sourceDto, TableCustomizerTable tableCustomizerTable) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerTable;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerTable);
		if (sourceDto.getOid() != null) {
			tableCustomizerTable.setOid(sourceDto.getOid());
		}
		tableCustomizerTable.setCodOwner(sourceDto.getCodOwner());
		tableCustomizerTable.setNumMandant(sourceDto.getNumMandant());
		tableCustomizerTable.setFipUser(sourceDto.getFipUser());
		tableCustomizerTable.setFipDate(sourceDto.getFipDate());
		tableCustomizerTable.setNamTable(sourceDto.getNamTable());
	    return tableCustomizerTable;
	}
			
    protected TableCustomizerUser mapFromTableCustomizerTableViewToTableCustomizerUser(TableCustomizerTableView sourceDto) {
    	return mapFromTableCustomizerTableViewToTableCustomizerUser(sourceDto, MapperContextHolder.createEntity(TableCustomizerUser.class));
    }

    protected TableCustomizerUser mapFromTableCustomizerTableViewToTableCustomizerUser(TableCustomizerTableView sourceDto, TableCustomizerUser tableCustomizerUser) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerUser;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerUser);
		if (sourceDto.getTableCustomizerUserOid() != null  && MapperContextHolder.isContext("tableCustomizerUser")) {
			tableCustomizerUser.setOid(sourceDto.getTableCustomizerUserOid());
		}
	    return tableCustomizerUser;
	}
	
	protected TableCustomizerTableView mapFromTableCustomizerTableToTableCustomizerTableView(TableCustomizerTable tableCustomizerTable) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return (TableCustomizerTableView) MapperContextHolder.getContext().get(tableCustomizerTable);
		}
		TableCustomizerTableView tableCustomizerTableView = new TableCustomizerTableView();
    	MapperContextHolder.getContext().put(tableCustomizerTable, tableCustomizerTableView);
		tableCustomizerTableView.setOid(tableCustomizerTable.getOid());
		tableCustomizerTableView.setCodOwner(tableCustomizerTable.getCodOwner());
		tableCustomizerTableView.setNumMandant(tableCustomizerTable.getNumMandant());
		tableCustomizerTableView.setFipUser(tableCustomizerTable.getFipUser());
		tableCustomizerTableView.setFipDate(tableCustomizerTable.getFipDate());
		tableCustomizerTableView.setNamTable(tableCustomizerTable.getNamTable());
        return tableCustomizerTableView;
	}
	
	protected TableCustomizerTableView mapFromTableCustomizerTableToTableCustomizerTableView(TableCustomizerTable tableCustomizerTable, TableCustomizerTableView tableCustomizerTableView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return tableCustomizerTableView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerTable, tableCustomizerTableView);
		tableCustomizerTableView.setOid(tableCustomizerTable.getOid());
		tableCustomizerTableView.setCodOwner(tableCustomizerTable.getCodOwner());
		tableCustomizerTableView.setNumMandant(tableCustomizerTable.getNumMandant());
		tableCustomizerTableView.setFipUser(tableCustomizerTable.getFipUser());
		tableCustomizerTableView.setFipDate(tableCustomizerTable.getFipDate());
		tableCustomizerTableView.setNamTable(tableCustomizerTable.getNamTable());
        return tableCustomizerTableView;
	}
	
	protected TableCustomizerTableView mapFromTableCustomizerUserToTableCustomizerTableView(TableCustomizerUser tableCustomizerUser) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return (TableCustomizerTableView) MapperContextHolder.getContext().get(tableCustomizerUser);
		}
		TableCustomizerTableView tableCustomizerTableView = new TableCustomizerTableView();
    	MapperContextHolder.getContext().put(tableCustomizerUser, tableCustomizerTableView);
		tableCustomizerTableView.setTableCustomizerUserOid(tableCustomizerUser.getOid());
        return tableCustomizerTableView;
	}
	
	protected TableCustomizerTableView mapFromTableCustomizerUserToTableCustomizerTableView(TableCustomizerUser tableCustomizerUser, TableCustomizerTableView tableCustomizerTableView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return tableCustomizerTableView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerUser, tableCustomizerTableView);
		tableCustomizerTableView.setTableCustomizerUserOid(tableCustomizerUser.getOid());
        return tableCustomizerTableView;
	}
	
	

	private DtoAssembler<TableCustomizerUserView> createDtoAssemblerTableCustomizerUserView() {
		return new DtoAssembler<TableCustomizerUserView>() {
		    public TableCustomizerUserView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		TableCustomizerUserView result = mapFromTableCustomizerUserToTableCustomizerUserView(tableCustomizerUser);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public TableCustomizerUserView mapFromEntity(Object sourceEntity, TableCustomizerUserView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		mapFromTableCustomizerUserToTableCustomizerUserView(tableCustomizerUser, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(TableCustomizerUserView sourceDto, Class<T> target) {
				if (TableCustomizerUserView.class.equals(target)) {
					TableCustomizerUserView result = new TableCustomizerUserView();
					mapFromTableCustomizerUserViewToTableCustomizerUserView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerUser.class.equals(target)) {
		    		return target.cast(mapFromTableCustomizerUserViewToTableCustomizerUser(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(TableCustomizerUserView sourceDto, Object targetObject) {
		    	if (targetObject instanceof TableCustomizerUserView) {
					mapFromTableCustomizerUserViewToTableCustomizerUserView(sourceDto, (TableCustomizerUserView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerUser) {
		    		mapFromTableCustomizerUserViewToTableCustomizerUser(sourceDto, (TableCustomizerUser)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromTableCustomizerUserViewToTableCustomizerUserView(TableCustomizerUserView sourceDto, TableCustomizerUserView tableCustomizerUserView) {
   		tableCustomizerUserView.setOid(sourceDto.getOid());
   		tableCustomizerUserView.setCodOwner(sourceDto.getCodOwner());
   		tableCustomizerUserView.setNumMandant(sourceDto.getNumMandant());
   		tableCustomizerUserView.setFipUser(sourceDto.getFipUser());
   		tableCustomizerUserView.setFipDate(sourceDto.getFipDate());
   		tableCustomizerUserView.setCodUser(sourceDto.getCodUser());
    }
    
    protected TableCustomizerUser mapFromTableCustomizerUserViewToTableCustomizerUser(TableCustomizerUserView sourceDto) {
    	return mapFromTableCustomizerUserViewToTableCustomizerUser(sourceDto, MapperContextHolder.createEntity(TableCustomizerUser.class));
    }

    protected TableCustomizerUser mapFromTableCustomizerUserViewToTableCustomizerUser(TableCustomizerUserView sourceDto, TableCustomizerUser tableCustomizerUser) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerUser;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerUser);
		if (sourceDto.getOid() != null) {
			tableCustomizerUser.setOid(sourceDto.getOid());
		}
		tableCustomizerUser.setCodOwner(sourceDto.getCodOwner());
		tableCustomizerUser.setNumMandant(sourceDto.getNumMandant());
		tableCustomizerUser.setFipUser(sourceDto.getFipUser());
		tableCustomizerUser.setFipDate(sourceDto.getFipDate());
		tableCustomizerUser.setCodUser(sourceDto.getCodUser());
	    return tableCustomizerUser;
	}
	
	protected TableCustomizerUserView mapFromTableCustomizerUserToTableCustomizerUserView(TableCustomizerUser tableCustomizerUser) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return (TableCustomizerUserView) MapperContextHolder.getContext().get(tableCustomizerUser);
		}
		TableCustomizerUserView tableCustomizerUserView = new TableCustomizerUserView();
    	MapperContextHolder.getContext().put(tableCustomizerUser, tableCustomizerUserView);
		tableCustomizerUserView.setOid(tableCustomizerUser.getOid());
		tableCustomizerUserView.setCodOwner(tableCustomizerUser.getCodOwner());
		tableCustomizerUserView.setNumMandant(tableCustomizerUser.getNumMandant());
		tableCustomizerUserView.setFipUser(tableCustomizerUser.getFipUser());
		tableCustomizerUserView.setFipDate(tableCustomizerUser.getFipDate());
		tableCustomizerUserView.setCodUser(tableCustomizerUser.getCodUser());
        return tableCustomizerUserView;
	}
	
	protected TableCustomizerUserView mapFromTableCustomizerUserToTableCustomizerUserView(TableCustomizerUser tableCustomizerUser, TableCustomizerUserView tableCustomizerUserView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return tableCustomizerUserView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerUser, tableCustomizerUserView);
		tableCustomizerUserView.setOid(tableCustomizerUser.getOid());
		tableCustomizerUserView.setCodOwner(tableCustomizerUser.getCodOwner());
		tableCustomizerUserView.setNumMandant(tableCustomizerUser.getNumMandant());
		tableCustomizerUserView.setFipUser(tableCustomizerUser.getFipUser());
		tableCustomizerUserView.setFipDate(tableCustomizerUser.getFipDate());
		tableCustomizerUserView.setCodUser(tableCustomizerUser.getCodUser());
        return tableCustomizerUserView;
	}
	
	

	private DtoAssembler<UserTableView> createDtoAssemblerUserTableView() {
		return new DtoAssembler<UserTableView>() {
		    public UserTableView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
		    	if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		UserTableView result = mapFromTableCustomizerUserToUserTableView(tableCustomizerUser);
		    		return result;
		    	} else if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		UserTableView result = mapFromTableCustomizerTableToUserTableView(tableCustomizerTable);
		    		return result;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public UserTableView mapFromEntity(Object sourceEntity, UserTableView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
		    	if (sourceEntity instanceof TableCustomizerUser) {
					TableCustomizerUser tableCustomizerUser = (TableCustomizerUser) sourceEntity;
		    		mapFromTableCustomizerUserToUserTableView(tableCustomizerUser, targetDto);
		    		return targetDto;
		    	} else if (sourceEntity instanceof TableCustomizerTable) {
					TableCustomizerTable tableCustomizerTable = (TableCustomizerTable) sourceEntity;
		    		mapFromTableCustomizerTableToUserTableView(tableCustomizerTable, targetDto);
		    		return targetDto;
		    	}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
		    	
			}
			
			public <T> T mapTo(UserTableView sourceDto, Class<T> target) {
				if (UserTableView.class.equals(target)) {
					UserTableView result = new UserTableView();
					mapFromUserTableViewToUserTableView(sourceDto, result);
					return target.cast(result);
				}		    	
				else if (TableCustomizerUser.class.equals(target)) {
		    		return target.cast(mapFromUserTableViewToTableCustomizerUser(sourceDto));
		    	}
				else if (TableCustomizerTable.class.equals(target)) {
		    		return target.cast(mapFromUserTableViewToTableCustomizerTable(sourceDto));
		    	}
		    	else {
		    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
		    	}
		    }

		    public void mapTo(UserTableView sourceDto, Object targetObject) {
		    	if (targetObject instanceof UserTableView) {
					mapFromUserTableViewToUserTableView(sourceDto, (UserTableView) targetObject);
				}				
		    	else if (targetObject instanceof TableCustomizerUser) {
		    		mapFromUserTableViewToTableCustomizerUser(sourceDto, (TableCustomizerUser)targetObject);
		    	}		    	
		    	else if (targetObject instanceof TableCustomizerTable) {
		    		mapFromUserTableViewToTableCustomizerTable(sourceDto, (TableCustomizerTable)targetObject);
		    	}		    	
		    	else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}

    protected void mapFromUserTableViewToUserTableView(UserTableView sourceDto, UserTableView userTableView) {
   		userTableView.setCodUser(sourceDto.getCodUser());
   		userTableView.setFipUser(sourceDto.getFipUser());
   		userTableView.setNumMandant(sourceDto.getNumMandant());
   		userTableView.setNamTable(sourceDto.getNamTable());
   		userTableView.setTableCustomizerColumns(sourceDto.getTableCustomizerColumns());
    }
    
    protected TableCustomizerUser mapFromUserTableViewToTableCustomizerUser(UserTableView sourceDto) {
    	return mapFromUserTableViewToTableCustomizerUser(sourceDto, MapperContextHolder.createEntity(TableCustomizerUser.class));
    }

    protected TableCustomizerUser mapFromUserTableViewToTableCustomizerUser(UserTableView sourceDto, TableCustomizerUser tableCustomizerUser) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerUser;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerUser);
		tableCustomizerUser.setCodUser(sourceDto.getCodUser());
		tableCustomizerUser.setFipUser(sourceDto.getFipUser());
		tableCustomizerUser.setNumMandant(sourceDto.getNumMandant());
	    return tableCustomizerUser;
	}
			
    protected TableCustomizerTable mapFromUserTableViewToTableCustomizerTable(UserTableView sourceDto) {
    	return mapFromUserTableViewToTableCustomizerTable(sourceDto, MapperContextHolder.createEntity(TableCustomizerTable.class));
    }

    protected TableCustomizerTable mapFromUserTableViewToTableCustomizerTable(UserTableView sourceDto, TableCustomizerTable tableCustomizerTable) {
    	if (MapperContextHolder.getContext().containsKey(sourceDto)) {
    		return tableCustomizerTable;
    	}
	    MapperContextHolder.getContext().put(sourceDto, tableCustomizerTable);
		tableCustomizerTable.setNamTable(sourceDto.getNamTable());
	    if (!MapperContextHolder.isContext("saveTableCustomizerTable") && !MapperContextHolder.isContext("updateTableCustomizerTable")) {
			if (null != sourceDto.getTableCustomizerColumns()) {
				for (TableCustomizerColumnView tableCustomizerColumnView : sourceDto.getTableCustomizerColumns()) {
					TableCustomizerColumn tableCustomizerColumns = (TableCustomizerColumn) dtoAssemblerRegistry.getDtoAssembler(TableCustomizerColumnView.class).mapTo(tableCustomizerColumnView, TableCustomizerColumn.class);
					tableCustomizerTable.addTableCustomizerColumns(tableCustomizerColumns);
				}
			}
	    }
	    return tableCustomizerTable;
	}
	
	protected UserTableView mapFromTableCustomizerUserToUserTableView(TableCustomizerUser tableCustomizerUser) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return (UserTableView) MapperContextHolder.getContext().get(tableCustomizerUser);
		}
		UserTableView userTableView = new UserTableView();
    	MapperContextHolder.getContext().put(tableCustomizerUser, userTableView);
		userTableView.setCodUser(tableCustomizerUser.getCodUser());
		userTableView.setFipUser(tableCustomizerUser.getFipUser());
		userTableView.setNumMandant(tableCustomizerUser.getNumMandant());
        return userTableView;
	}
	
	protected UserTableView mapFromTableCustomizerUserToUserTableView(TableCustomizerUser tableCustomizerUser, UserTableView userTableView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerUser)) {
			return userTableView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerUser, userTableView);
		userTableView.setCodUser(tableCustomizerUser.getCodUser());
		userTableView.setFipUser(tableCustomizerUser.getFipUser());
		userTableView.setNumMandant(tableCustomizerUser.getNumMandant());
        return userTableView;
	}
	
	protected UserTableView mapFromTableCustomizerTableToUserTableView(TableCustomizerTable tableCustomizerTable) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return (UserTableView) MapperContextHolder.getContext().get(tableCustomizerTable);
		}
		UserTableView userTableView = new UserTableView();
    	MapperContextHolder.getContext().put(tableCustomizerTable, userTableView);
		userTableView.setNamTable(tableCustomizerTable.getNamTable());
		if (tableCustomizerTable.hasTableCustomizerColumns()) {
			for (TableCustomizerColumn next : tableCustomizerTable.getTableCustomizerColumns()) {
	    		TableCustomizerColumnView nextTableCustomizerColumnView = (TableCustomizerColumnView) MapperContextHolder.getContext().get(next);
	    		if (null == nextTableCustomizerColumnView) {
		        	nextTableCustomizerColumnView = dtoAssemblerRegistry.getDtoAssembler(TableCustomizerColumnView.class).mapFromEntity(next);
		        }
	        	userTableView.addTableCustomizerColumns(nextTableCustomizerColumnView);
	    	}
		}
        return userTableView;
	}
	
	protected UserTableView mapFromTableCustomizerTableToUserTableView(TableCustomizerTable tableCustomizerTable, UserTableView userTableView) {
		if (MapperContextHolder.getContext().containsKey(tableCustomizerTable)) {
			return userTableView;
		}		
    	MapperContextHolder.getContext().put(tableCustomizerTable, userTableView);
		userTableView.setNamTable(tableCustomizerTable.getNamTable());
		if (tableCustomizerTable.hasTableCustomizerColumns()) {
			for (TableCustomizerColumn next : tableCustomizerTable.getTableCustomizerColumns()) {
	    		TableCustomizerColumnView nextTableCustomizerColumnView = (TableCustomizerColumnView) MapperContextHolder.getContext().get(next);
	    		if (null == nextTableCustomizerColumnView) {
		        	nextTableCustomizerColumnView = dtoAssemblerRegistry.getDtoAssembler(TableCustomizerColumnView.class).mapFromEntity(next);
		        }
	        	userTableView.addTableCustomizerColumns(nextTableCustomizerColumnView);
	    	}
		}
        return userTableView;
	}
	
	

}
