/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.addons.ui.table.customizer.mdl.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.openxma.addons.ui.table.customizer.common.CommonColumnModel;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.CommonTableModel;
import org.openxma.addons.ui.table.customizer.common.CommonUserModel;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.CodDatatype;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.FilterOperator;
import org.openxma.addons.ui.table.customizer.mdl.dao.TableCustomizerTableDao;
import org.openxma.addons.ui.table.customizer.mdl.dao.TableCustomizerUserDao;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerColumnView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerFilterView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerTableView;
import org.openxma.addons.ui.table.customizer.mdl.dto.TableCustomizerUserView;
import org.openxma.addons.ui.table.customizer.mdl.dto.UserTableView;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerColumn;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerFilter;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerTable;
import org.openxma.addons.ui.table.customizer.mdl.model.TableCustomizerUser;
import org.openxma.addons.ui.table.customizer.mdl.service.TableCustomizerDas;
import org.openxma.addons.ui.table.customizer.mdl.service.TableCustomizerService;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service("tableCustomizerService")
@Transactional
public class TableCustomizerServiceImpl implements TableCustomizerService {

    @Autowired
    private TableCustomizerUserDao userDao;

    @Autowired
    private TableCustomizerDas das;

    @Autowired
    private TableCustomizerTableDao tableDao;

    @Autowired
    private Mapper mapper;

//    public void appendTableForUser(TableCustomizerTableView tableView,
//            TableCustomizerUserView userView) {
//        TableCustomizerUser user = userDao.findUser(userView.getCodUser(),
//                userView.getNumMandant());
//        if (user == null) {
//            user = userDao.createAndSave(userView.getNumMandant(), userView
//                    .getCodUser(), userView.getCodUser());
//        }
//        TableCustomizerTable table = tableDao.findTableForUser(userView
//                .getCodUser(), userView.getNumMandant(), tableView
//                .getNamTable());
//        if (table == null) {
//            table = mapper.createAndMapOne(tableView,
//                    TableCustomizerTable.class);
//            user.addTableCustomizerTables(table);
//        }
//    }

    public TableCustomizerUserView findUser(String codName, Integer numMandant) {
        return das.findUser(codName, numMandant);
    }

    public TableCustomizerTableView findTableForUser(String codUser,
            Integer mandant, String tableName) {
        return das.findTableForUser(codUser, mandant, tableName);
    }

    public UserTableView findTableViewForUser(String codUser, Integer mandant,
            String tableName) {
        TableCustomizerUser user = userDao.findUser(codUser, mandant);
        if (user == null) {
            return null;
        } else {
            UserTableView view = new UserTableView();
            mapper.mapOne(user, view);
            TableCustomizerTable table = tableDao.findTableForUser(codUser,
                    mandant, tableName);
            if (table != null) {
                mapper.mapOne(table, view);
            }
            return view;
        }
    }


    public CommonUserModel getUserModel(String user, int mandant) {
        TableCustomizerUser userSettings = userDao.findUser(user, mandant);
        if (userSettings == null) {
            return null;
        } else {
            CommonUserModel userModel = new CommonUserModel();
            userModel.setCodUser(userSettings.getCodUser());
            userModel.setNumMandant(userSettings.getNumMandant());
            for (TableCustomizerTable table : userSettings.getTableCustomizerTables()) {
                CommonTableModel ctm = new CommonTableModel();
                ctm.setNamTable(table.getNamTable());
                ctm.setAppCode(table.getCodOwner());
                for (TableCustomizerColumn cv : table.getTableCustomizerColumns()) {
                    CommonColumnModel ccm = new CommonColumnModel();
                    ccm.setAscending(cv.getBolAscending());
                    ccm.setNamColumn(cv.getNamColumn());
                    ccm.setDefPercent(cv.getRatDefault()==null?null:cv.getRatDefault().floatValue());
                    ccm.setResizeable(cv.getBolIsResizable());
                    ccm.setDefWith(cv.getNumDefaultWidth());
                    ccm.setDefMinWith(cv.getNumDefaultMinWith());
                    ccm.setAndFilter(cv.getBolAndFilter());
                    ccm.setNullableFilter(cv.getBolIsNullableFilter());
                    for (TableCustomizerFilter filter : cv.getTableCustomizerFilter()) {
                        CommonFilterModel cfm =
                            new CommonFilterModel(
                                    FilterOperator.getFilterOperatorByNormalizedName(filter.getCodFilterOperator()),
                                    filter.getTxtValue());
                        ccm.addFilterModel(cfm);

                    }
                    ccm.setCodDataType(CodDatatype.parse(cv.getCodDataType()));
                    ccm.setTxtFormatterPattern(cv.getTxtFormatterPattern());
                    ccm.setNumColumnIndex(cv.getNumColumnIndex());
                    ccm.setNumOrder(cv.getNumOrder());
                    ccm.setSortOrder(cv.getNumSort());
                    ccm.setTxtColumn(cv.getTxtColumn());
                    ccm.setVisible(cv.getBolVisible());
                    ctm.addColumnModel(ccm);
                }
                userModel.setTableModel(ctm);
            }
            return userModel;
        }
    }

    public void persist(CommonTableModel tableModel, String user, int mandant){
        TableCustomizerUser findUser = userDao.findUser(user,mandant);
        if (findUser == null) {
            findUser = userDao.createAndSave(tableModel.getAppCode(),mandant, user, user);
        }
        TableCustomizerTable table = tableDao.findTableForUser(user, mandant, tableModel.getNamTable());
        if (table == null) {
            TableCustomizerTableView tv = new TableCustomizerTableView();
            tv.setFipUser(user);
            tv.setNamTable(tableModel.getNamTable());
            tv.setNumMandant(mandant);
            tv.setCodOwner(tableModel.getAppCode());
            table = mapper.createAndMapOne(tv,TableCustomizerTable.class);
            findUser.addTableCustomizerTables(table);
        }

        //TODO:
        // if finduser == null
        //   neuen user anlegen
        // if finduser.gettable(fqTableNmae)==null
        //   neue table settings speichern
        // else
        //   alte settings aktualisieren



        List<TableCustomizerColumnView> cols = new ArrayList<TableCustomizerColumnView>();
        for (CommonColumnModel cm : tableModel.getColumnModels()) {
            TableCustomizerColumnView cView = new TableCustomizerColumnView();
            cView.setNamColumn(cm.getNamColumn());
            cView.setFipUser(user);
            Float defPercent = cm.getDefPercent();
            cView.setRatDefault(defPercent == null? null:new BigDecimal(defPercent));
            cView.setNumDefaultWidth(cm.getDefWith());
            cView.setNumDefaultMinWith(cm.getDefMinWith());
            cView.setBolIsResizable(cm.isResizeable());
            cView.setCodOwner(tableModel.getAppCode());
            cView.setNumMandant(mandant);
            cView.setNumOrder(cm.getNumOrder());
            cView.setNumSort(cm.getSortOrder());
            cView.setNumColumnIndex(cm.getNumColumnIndex());
            cView.setTxtColumn(cm.getTxtColumn());
            cView.setCodDataType(cm.getCodDataType()==null?null:cm.getCodDataType().name());
            cView.setBolAndFilter(cm.isAndFilter());
            cView.setBolIsNullableFilter(cm.isNullableFilter());
            List<CommonFilterModel> filterModel = cm.getFilterModel();
            for (CommonFilterModel cfm : filterModel) {
                TableCustomizerFilterView fv = new TableCustomizerFilterView();
                fv.setCodFilterOperator(cfm.getCodFilterOperator().name());
                fv.setNumMandant(mandant);
                fv.setTxtValue(cfm.getTxtValue());
                fv.setFipUser(user);
                fv.setCodOwner(tableModel.getAppCode());
                cView.addTableCustomizerFilter(fv);
                cView.setCodDataType(cm.getCodDataType()==null?null:cm.getCodDataType().name());
            }
            cView.setTxtFormatterPattern(cm.getTxtFormatterPattern());
            cView.setBolAscending(cm.isAscending());
            cView.setBolVisible(cm.isVisible());
            cols.add(cView);

        }
        table.removeAllTableCustomizerColumns();
        List<TableCustomizerColumn> mappedCols = mapper.createAndMapMany(cols, TableCustomizerColumn.class);
        for (TableCustomizerColumn col : mappedCols) {
            table.addTableCustomizerColumns(col);
        }
        findUser.addTableCustomizerTables(table);
    }

    public CommonTableModel getTableModel(String user, int mandant, String fqTableName) {
        TableCustomizerTable table = tableDao.findTableForUser(user, mandant, fqTableName);
        if (table == null) return null;
        CommonTableModel ctm = new CommonTableModel();
        ctm.setNamTable(table.getNamTable());
        ctm.setAppCode(table.getCodOwner());
        for (TableCustomizerColumn cv : table.getTableCustomizerColumns()) {
            CommonColumnModel ccm = new CommonColumnModel();
            ccm.setAscending(cv.getBolAscending());
            ccm.setNamColumn(cv.getNamColumn());
            ccm.setNullableFilter(cv.getBolIsNullableFilter());
            ccm.setDefPercent(cv.getRatDefault() == null ? null : cv.getRatDefault().floatValue());
            ccm.setResizeable(cv.getBolIsResizable());
            ccm.setDefWith(cv.getNumDefaultWidth());
            ccm.setDefMinWith(cv.getNumDefaultMinWith());
            ccm.setAndFilter(cv.getBolAndFilter());
            for (TableCustomizerFilter filter : cv.getTableCustomizerFilter()) {
                CommonFilterModel cfm = new CommonFilterModel(FilterOperator
                        .getFilterOperatorByNormalizedName(filter.getCodFilterOperator()),
                        filter.getTxtValue());
                ccm.addFilterModel(cfm);

            }
            ccm.setTxtFormatterPattern(cv.getTxtFormatterPattern());
            ccm.setCodDataType(CodDatatype.parse(cv.getCodDataType()));
            ccm.setNumColumnIndex(cv.getNumColumnIndex());
            ccm.setNumOrder(cv.getNumOrder());
            ccm.setSortOrder(cv.getNumSort());
            ccm.setTxtColumn(cv.getTxtColumn());
            ccm.setVisible(cv.getBolVisible());
            ctm.addColumnModel(ccm);
        }
        return ctm;
    }









}
