package org.openxma.addons.ui.table.customizer.xma.client;

import java.util.Locale;
import java.util.ResourceBundle;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import at.spardat.xma.component.*;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.graphics.Point;
import at.spardat.xma.page.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.simple.*;

/**
 * This class implements the generic client part of FilterPage.
 * It implements the widgets and models of the page.
 *
 */
public abstract class FilterPageGen extends DialogPage {
    private Locale genPageLocale = null;
    private ResourceBundle genPageMessages = null;
    Button applyFilterButtonW;
    Button cancelButtonW;
    ScrolledComposite scrollCompositeW;
    Composite scrollCompositeinnerCompW;
    Button isNullableW;
    ISimpleWMClient isNullable;
    WModel[] widgetModels;
    Control[] widgets;

    /**
     * Constructs the FilterPageGen.
     * @param parent the parent page
     */
    public FilterPageGen(PageClient parent) {
        super(parent,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the FilterPageGen.
     * @param component the component this page belongs to
     */
    public FilterPageGen(ComponentClient component) {
        super(component,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the FilterPageGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     */
    public FilterPageGen(ComponentClient component, Shell parentShell) {
        super(component,parentShell,false,SWT.PRIMARY_MODAL|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }

    /**
     * Constructs the FilterPageGen.
     * @param component the component this page belongs to
     * @param parentShell the shell of the calling dialog
     * @param modal the SWT-style for the desired modality
     */
    public FilterPageGen(ComponentClient component, Shell parentShell, int modal) {
        super(component,parentShell,false,modal|SWT.CLOSE|SWT.RESIZE|SWT.TITLE);
        createModels();
    }


    /*
     * returns the Locale used for this page
    */
    final protected Locale getGenPageLocale() {
        if (genPageLocale == null) {
            genPageLocale = getComponent().getContext().getLocale();
        }
        return genPageLocale;
    }

    /*
     * returns the message resource bundle used for this page
    */
    final protected ResourceBundle getGenPageMessages() {
        if (genPageMessages == null) {
            genPageMessages = ResourceBundle.getBundle("org.openxma.addons.ui.table.customizer.xma.client.TableCustomizerComp",getGenPageLocale());
        }
        return genPageMessages;
    }

    /**
     * Gives the URI of the help for this page
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this page
     */
    public String getHelpUri() {
        return "help/org/openxma/addons/ui/table/customizer/xma/FilterPage.html";
    }

    /**
     * Create and initialize the WidgetModels of the page.
     * Subpages which are not lazy are created too.
     */
    public void createModels() {
        isNullable = new SimpleWMClient((short) 0, Types.T_BOOLEAN, this);

        widgetModels = new WModel[] { (WModel) isNullable };
        getComponent().registerPageModel(this);
    }

    /**
     * Indicate if the WidgetModels are already created.
     * @return true if the WidgetModels exist.
     */
    public boolean hasModels() {
        return widgetModels != null;
    }

    /**
     * Remove all WidgetModels of the page.
     */
    public void removeWidgetModels() {
        isNullable = null;
        widgetModels = null;
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return TableCustomizerComp.FilterPageClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public TableCustomizerComp getTypedComponent() {
        return (TableCustomizerComp) getComponent();
    }

    /**
     * Returns an array of all Widgets of this page.
     * @return array of Widgets. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public Control[] getWidgets() {
        return widgets;
    }

    /**
     * Replaces the widget array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWidgets(Control[] widgets) {
        this.widgets=widgets;
    }

    /**
     * Create and initialize all SWT-Widgets of the page.
     * It <br>
     * <ul>
     * <li> instantiates the Widgets </li>
     * <li> sets constant texts (Title, Labels, Tooltips) and Images </li>
     * <li> registers the eventadapter </li>
     * <li> defines the Layout (Attachments, Distances, Sizes) </li>
     * <li> sets the focus </li>
     * <li> defines the Taborder </li>
     * </ul>
     */
    protected void createWidgets() {
        FormData data;
        Control[] tabOrder;
        EventAdapter adapter = new EventAdapter(this);
        final Scaler scaler = Scaler.getInstance(getComposite());

        final Shell shell = getShell();
        if(getDialog()==this) {
            shell.setText(getGenPageMessages().getString("FilterPage"));
            shell.addHelpListener(adapter);
        } else {
            getComposite().addHelpListener(adapter);
        }
        Composite filterPageCompW = getComposite();
        FormLayout layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        filterPageCompW.setLayout(layout);

        applyFilterButtonW = new Button(filterPageCompW, SWT.PUSH|SWT.CENTER);
        applyFilterButtonW.setText(getGenPageMessages().getString("FilterPage.applyFilterButtonW"));
        applyFilterButtonW.setData("WIDGET_ID","applyFilterButtonW");
        applyFilterButtonW.addSelectionListener(adapter);

        cancelButtonW = new Button(filterPageCompW, SWT.PUSH|SWT.CENTER);
        cancelButtonW.setText(getGenPageMessages().getString("FilterPage.cancelButtonW"));
        cancelButtonW.setData("WIDGET_ID","cancelButtonW");
        cancelButtonW.addSelectionListener(adapter);

        scrollCompositeW = new ScrolledComposite(filterPageCompW,SWT.NONE|SWT.H_SCROLL|SWT.V_SCROLL);
        scrollCompositeW.setExpandHorizontal(true);
        scrollCompositeW.setExpandVertical(true);
        scrollCompositeW.setData("WIDGET_ID","scrollCompositeW");

        scrollCompositeinnerCompW = new Composite(scrollCompositeW,SWT.NONE);
        layout = new FormLayout();
        layout.marginHeight = scaler.convertYToCurrent(3);
        layout.marginWidth = scaler.convertXToCurrent(3);
        scrollCompositeinnerCompW.setLayout(layout);
        scrollCompositeinnerCompW.setData("WIDGET_ID","scrollCompositeinnerCompW");

        isNullableW = new Button(filterPageCompW, SWT.CHECK|SWT.CENTER);
        isNullableW.setText(getGenPageMessages().getString("FilterPage.isNullableW"));
        isNullableW.setToolTipText(getGenPageMessages().getString("FilterPage.isNullableWTip"));
        isNullableW.setData("WIDGET_ID","isNullableW");
        isNullableW.addSelectionListener(adapter);

        widgets = new Control[] { applyFilterButtonW, cancelButtonW, scrollCompositeW, scrollCompositeinnerCompW, isNullableW };

        data = new FormData();
        data.width = scaler.convertXToCurrent(100);
        data.right = new FormAttachment(cancelButtonW,scaler.convertXToCurrent(-3),SWT.LEFT);
        data.top = new FormAttachment(100,100,scaler.convertYToCurrent(-25));
        applyFilterButtonW.setLayoutData(data);

        data = new FormData();
        data.width = scaler.convertXToCurrent(100);
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(-3));
        data.top = new FormAttachment(applyFilterButtonW,scaler.convertYToCurrent(0),SWT.CENTER);
        cancelButtonW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        data.bottom = new FormAttachment(applyFilterButtonW,scaler.convertYToCurrent(-3),SWT.TOP);
        scrollCompositeW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(0));
        data.right = new FormAttachment(100,100,scaler.convertXToCurrent(0));
        data.top = new FormAttachment(0,100,scaler.convertYToCurrent(0));
        scrollCompositeinnerCompW.setLayoutData(data);

        data = new FormData();
        data.left = new FormAttachment(0,100,scaler.convertXToCurrent(3));
        data.top = new FormAttachment(applyFilterButtonW,scaler.convertYToCurrent(0),SWT.CENTER);
        isNullableW.setLayoutData(data);


        scrollCompositeW.setContent(scrollCompositeinnerCompW);
        Point size = scrollCompositeinnerCompW.computeSize(SWT.DEFAULT,SWT.DEFAULT);
        scrollCompositeW.setMinSize(size);

        if(getDialog()==this) {
            shell.setTabList(new Control[] { filterPageCompW });
        }
        tabOrder = new Control[4];
        tabOrder[0] = applyFilterButtonW;
        tabOrder[1] = cancelButtonW;
        tabOrder[2] = scrollCompositeW;
        tabOrder[3] = isNullableW;
        filterPageCompW.setTabList(tabOrder);
        tabOrder = new Control[0];
        scrollCompositeinnerCompW.setTabList(tabOrder);
        if(getDialog()==this) {
            shell.setSize(scaler.convertXToCurrent(450), scaler.convertYToCurrent(180));
        }
    }

    /**
     * Remove all Widgets of the page after they have been disposed.
     */
    public void removeWidgets() {
        applyFilterButtonW = null;
        cancelButtonW = null;
        scrollCompositeW = null;
        scrollCompositeinnerCompW = null;
        isNullableW = null;
        widgets = null;
    }


    /**
     * Attaches all WidgetModels of the page to there corresponding Widgets.
     */
    public void attachUIImpl() throws AttachmentExceptionClient {
        isNullableW.setData(isNullable.getUIDelegate());
        isNullable.getUIDelegate().attachUI(isNullableW,null);
    }

    /**
     * This method will be called every time the FilterPage becomes visible.
     */
    protected void enter() {
        super.enter();
    }

    /**
     * Calls the clientevent configured for the widget of the event.
     *
     * @param event the SWT-Event that happened.
     * @param type the type of the Event, as defined by the event type constants in class SWT.
     */
    protected void clientEvent(SelectionEvent event,int type) {
        if (event.widget == applyFilterButtonW) {
            applyFilterButtonWEvent();
        } else if (event.widget == cancelButtonW) {
            cancelButtonWEvent();
        }
    }

    /**
     * This method will be called every time the user clicks on applyFilterButtonW.
     */
    abstract protected void applyFilterButtonWEvent();

    /**
     * This method will be called every time the user clicks on cancelButtonW.
     */
    abstract protected void cancelButtonWEvent();

}
