package org.openxma.addons.ui.table.customizer.xma.client;

import java.util.List;
import java.util.Map;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Table;
import org.openxma.addons.ui.table.customizer.common.ColumnMetaData;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.CommonTableModel;

import at.spardat.xma.mdl.paging.Customizer;
import at.spardat.xma.mdl.paging.PagingWMClient;
import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.mdl.table.TableLayoutManager;
import at.spardat.xma.mdl.table.TableWMClient;

public interface ITableCustomizer extends ITableCustomizerComp, Customizer {

    final static int COLUMN_DISPLAY_TEXT = 0;
    final static int COLUMN_INDEX = 1;

    /**
     * this setter must be invoked prior to calling the customize() method
     *
     * @param tlm the {@link TableLayoutManager} this customizer belongs to.
     *
     * @see Customizer#customize()
     */
    void setTableLayoutManager(TableLayoutManager tlm);

    /**
     * getter for the tableLayoutManager
     *
     * @return
     * @since 1.0
     *
     */
    TableLayoutManager getTableLayoutManager();



    /**
     * this setter must be invoked prior to calling the customize() method
     *
     * @param tableWMClient
     *
     * @see Customizer#customize()
     */
    void setTableWMClient(ITableWMClient tableWMClient);


    /**
     *
     * @return ITableWMClient
     * @since 1.0
     */
    ITableWMClient getTableWMClient();


    /**
     * this name is used to persist table settings
     * since a tablename may not be unique within your aplication,
     * give here a fully qualified name of the table by
     * entering the following concatanation of strings
     * &lt;package-of-page>_&lt;pagename-of-table>_&lt;table-name>
     * <p/>example. lets say your table name is "testtable" and it is used in
     * your client page called "TableClientPage".<br>so a correct call would be:<br>
     * <code>setFQTableName(TableClientPage.getClass().getName() +"_"+ "testtable")</code>
     *
     * <p>be aware of the length of the full qualified name: it mut be less than 255 characters
     *
     * @param String
     * @see #getFQTableName()
     */
    void setFQTableName(String fullQualifiedTableName);

    String getFQTableName();

    void setPagingWMClient(PagingWMClient pagingWMClient);

    PagingWMClient getPagingWMClient();




    /**
     * returns the swt widget table which is fetched from the uideligate in the table model {@link #getTableWMClient()}
     * so no setTable method exist
     *
     * @return
     * @since 1.0
     *
     * @see TableWMClient#getUIDelegate()
     *
     */
    Table getTable();


    /**
     * applies table settings if a persisted setting for the current table is found
     *
     * @return true if setting was found and could be applied, false otherwise
     */
    boolean applySettings();


    /**
     * set a key-value map of all columns of the table that is customizable
     * where
     * => key represents the attribute-name (i.e. the name that is used for the filter query)
     * => value represents the display-name (label) of the column.
     *
     * this setter must be called only within the initialization process
     *
     * @param columnMap
     */
    void setColumnMap(Map<String, ColumnMetaData> columnMap);


    /**
     * returns the columnsMap as defined by the related setter method
     * @see #setColumnMap(Map)
     *
     * @return
     */
    Map<String, ColumnMetaData> getColumnMap();

    String getColumnAttributeByLabel(String columnLabel);

    void init();
    
    void register();

    short getId();

    /**
     * sets the application name up to 30 characters
     * must be unique name among all projects using the customizer
     * throws an illlegalArgumentException if val is longer than 30 characters
     *
     */
    void setAppCode(String appCode);

    /**
     * returns the appcode that wos set by {@link #setAppCode()}
     */
    String getAppCode();



    /**
     * setter to set a filter on a column programmatically. 
     * This new filter is not persisted.    
     *
     * @param fqTableName - the full qualified tablename. it can be retrieved from the generated page where the customizable table is defined
     * @param columnId  - the xma model id for the column the filter is to be applied
     * @param isAnd - if true all filters in the third parameter are joined with the "AND" operator<br>
     * otherwise the "OR" operator is used
     * @param filters a set of filters
     *
     * @see CommonFilterModel
     */
    void setFilter(String fqTableName, int columnId, boolean isAnd, List<CommonFilterModel> filters);
    
    /**
     * removes all filters for the table in the session. persistes filters remain in the db
     * 
     * @since 4.0.0     
     */
    void removeAllFilter();
    
    
    /**
     * 
     * @return
     * @since 4.0.0
     * 
     */
    CommonTableModel getTableModel();


}
