package org.openxma.addons.ui.table.customizer.xma.client;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.openxma.addons.ui.table.customizer.common.ColumnMetaData;
import org.openxma.addons.ui.table.customizer.common.CommonColumnModel;
import org.openxma.addons.ui.table.customizer.common.CommonDataModel.CodDatatype;
import org.openxma.addons.ui.table.customizer.common.CommonFilterModel;
import org.openxma.addons.ui.table.customizer.common.CommonTableModel;

import at.spardat.enterprise.fmt.ABcdFmtDefault;
import at.spardat.enterprise.fmt.ABcdFmtPattern;
import at.spardat.enterprise.fmt.ABcdFmtRange;
import at.spardat.enterprise.fmt.ADateFmtMediumSmart;
import at.spardat.enterprise.fmt.IFmt;
import at.spardat.xma.mdl.paging.PagingWMClient;
import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.mdl.table.TableLayoutManager;
import at.spardat.xma.mdl.table.TableUIDelegateClient;
import at.spardat.xma.mdl.table.XMATableColumn;
import at.spardat.xma.rpc.RemoteCallClient;
import at.spardat.xma.rpc.RemoteReply;
import at.spardat.xma.session.XMASessionClient;

/**
 * This class implements the client part of TableCustomizerComp.
 *
 */
public class TableCustomizerComp extends TableCustomizerCompGen implements ITableCustomizer {
    private TableLayoutManager tlm;
    private ITableWMClient tableWMClient;
    private String tableName;
    private Map<String, ColumnMetaData> columnMap = new HashMap<String, ColumnMetaData>();
    private PagingWMClient pagingWMClient;
    private String appCode;
    private CommonTableModel commonTableModel;


    /**
     * Initializes the TableCustomizerComp. This method is called by the
     * XMA-Runtime.
     *
     * @param session
     *            the XMASession, this TableCustomizerComp belongs to.
     */
    public TableCustomizerComp(XMASessionClient session) {
        super(session);
        setMainPage(new TableCustomizerPage(this));
    }

    /**
     * Invokes the TableCustomizerComp. This method reads the key-value-pairs of
     * the input, shows the windows of the TableCustomizerComp and returns the
     * output as key-value-pairs. This method is called by the XMA-Runtime.
     *
     * @param parentComposite
     *            a composite of the calling page. In most cases it will be a
     *            shell. It my be null.
     */
    public void invoke(Composite parentComposite) {
        super.invoke(parentComposite);
    }

    public boolean applySettings() {
        if (commonTableModel == null) {
            commonTableModel = commonModelFromTable(tableName);
        }
        
        applyTableModelOnUI(commonTableModel);
        return true;        
        
    }

    /**
     * this method applies the visibility/order settings on the GUI.
     * if the user resized any column widths on the table, the table model is updated
     * regarding to the new column sizes. (MANTIS 45565)
     * 
     * @param tableModel
     * @author s4162
     */
    protected void applyTableModelOnUI(CommonTableModel tableModel) {
        commonTableModel = tableModel;
        for (int i = 0; i < tableModel.getColumnModels().size(); i++) {

            CommonColumnModel cm = tableModel.getColumnModels().get(i);
            Integer numColumnIndex = cm.getNumColumnIndex();            
            XMATableColumn column = getTableWMClient().getColumn(numColumnIndex);
            int uiIndex = getTableWMClient().modelColumnIndexToUIColumnIndex(numColumnIndex);

            if (cm.getDefWith() == null || cm.getDefWith() == -1) {
                getTableLayoutManager().setMinWidth(uiIndex, cm.getDefMinWith());
                getTableLayoutManager().setPercent(uiIndex, cm.getDefPercent());
            } else {
                getTableLayoutManager().setAbsolutWidth(uiIndex, cm.getDefWith());
            }
            if (cm.isVisible()) {
                column.setVisible(cm.isVisible());
                getTableLayoutManager().layout();
            } else {
                getTableLayoutManager().collapseColumn(uiIndex);
            }

        }
        // int[] columnOrder = tableModel.getColumnOrder();
        int[] currentSwtOrder = getTable().getColumnOrder();
        int[] finalOrder = new int[currentSwtOrder.length];

        List<CommonColumnModel> cms = tableModel.getColumnModels();
        Map<Integer, Boolean> metaMap = new HashMap<Integer, Boolean>();
        for (int i = 0; i < cms.size(); i++) {
            CommonColumnModel columnModelByColIndex = cms.get(i);
            int swtColumnOrder = getTableWMClient().modelColumnIndexToUIColumnIndex(columnModelByColIndex.getNumOrder());
            finalOrder[swtColumnOrder] = getTableWMClient().modelColumnIndexToUIColumnIndex(columnModelByColIndex.getNumColumnIndex());
            metaMap.put(swtColumnOrder, Boolean.TRUE);
        }
        for (int i = 0; i < finalOrder.length; i++) {
            if (metaMap.get(i)==null) {
                finalOrder[i] = currentSwtOrder[i];
            }
        }

        getTable().setColumnOrder(finalOrder);
    }

    public void setTableLayoutManager(TableLayoutManager tlm) {
        this.tlm = tlm;
    }

    public void setTableWMClient(ITableWMClient tableWMClient) {
        this.tableWMClient = tableWMClient;
    }

    public void customize() {
        try {
            invoke(getComposite());
        } finally {
            // dispose();
        }
    }

    public TableLayoutManager getTableLayoutManager() {
        return tlm;
    }

    public Table getTable() {
        return (Table) (getTableUIDeligate()).getUIControl();
    }

    public ITableWMClient getTableWMClient() {
        return tableWMClient;
    }

    protected at.spardat.xma.mdl.table.TableUIDelegateClient getTableUIDeligate() {
        return (TableUIDelegateClient) getTableWMClient().getUIDelegate();
    }

    public String getFQTableName() {
        return tableName;
    }

    public void setFQTableName(String tableName) {
        this.tableName = tableName;
        setFQTableNameProperty(tableName);
    }

    public void setColumnMap(Map<String, ColumnMetaData> columnMap) {
        this.columnMap = columnMap;
    }

    public Map<String, ColumnMetaData> getColumnMap() {
        return columnMap;
    }

    public void register() {
        newRemoteCall("register").execute();
        
    }
    public void init() {
        RemoteCallClient newRemoteCall = newRemoteCall("init");
        newRemoteCall.setParameter(0, getColumnMap());
        newRemoteCall.setParameter(1, commonModelFromTable(getFQTableName()));
        RemoteReply reply = newRemoteCall.execute();        
        commonTableModel= (CommonTableModel) reply.getParameter(0);    
        
    }

    public String getColumnAttributeByLabel(String columnLabel) {
        for (Entry<String, ColumnMetaData> entry : getColumnMap().entrySet()) {
            String displayText = entry.getValue().getDisplayText();
            if (columnLabel.equals(displayText)) {
                return entry.getKey();
            }
        }
        throw new IllegalArgumentException("the given label value is not defined in the column mappings: "
                + columnLabel);
    }
    
    protected void updateTableModelColumnSizesFromUI(CommonTableModel commonTableModel){
        this.commonTableModel = commonTableModel;
        if (commonTableModel!= null) {
            for (int i = 0; i < commonTableModel.getColumnModels().size(); i++) {

                CommonColumnModel cm = commonTableModel.getColumnModels().get(i);
                Integer numColumnIndex = cm.getNumColumnIndex();            
                XMATableColumn column = getTableWMClient().getColumn(numColumnIndex);
                int uiIndex = getTableWMClient().modelColumnIndexToUIColumnIndex(numColumnIndex);

                if (cm.getDefWith() == null || cm.getDefWith() == -1) {
                    cm.setDefPercent(getTableLayoutManager().getPercent(uiIndex));
                    cm.setDefMinWith(getTableLayoutManager().getMinWidth(uiIndex));
                } else {
                    cm.setDefWith(getTable().getColumn(uiIndex).getWidth());
                    //cm.setDefWith(getTableLayoutManager().getAbsolutWidth(uiIndex));
                }
                if (cm.isVisible()) {
                    column.setVisible(cm.isVisible());                 
                } else {
                    getTableLayoutManager().collapseColumn(uiIndex);
                }
            }
        }
    }

    protected CommonTableModel commonModelFromTable(String fqTableName) {
        CommonTableModel ctm = new CommonTableModel();
        ctm.setNamTable(fqTableName);
        ctm.setAppCode(getAppCode());
        // load all column labels into model
        Map<String, ColumnMetaData> columnMap = getColumnMap();
        Iterator<String> colIterator = columnMap.keySet().iterator();
        while (colIterator.hasNext()) {
            String attrKey = colIterator.next();
            String attrLabel = (String) columnMap.get(attrKey).getDisplayText();
            String attrTypename = (String) columnMap.get(attrKey).getTypeName();
            for (int i = 0; i < getTableWMClient().getColumnCount(); i++) {
                int swtIndex = getTableWMClient().modelColumnIndexToUIColumnIndex(i);
                if (swtIndex == -1) {
                    // not a column in the widget, do not handle this column in
                    // the dialog
                } else {
                    TableColumn column = getTable().getColumn(swtIndex);
                    int colIndexInTableModel = i;
                    if (column.getText().equals(attrLabel)) {
                        // column at wiget index i found
                        String columnAttributeByLabel = getColumnAttributeByLabel(column.getText());
                        CommonColumnModel cmdl = new CommonColumnModel();
                        cmdl.setCodDataType(CodDatatype.getCodDatatypeByNativeName(attrTypename));
                        cmdl.setNamColumn(columnAttributeByLabel);
                        cmdl.setDefPercent(getTableLayoutManager().getPercent(swtIndex));
                        cmdl.setDefWith(getTableLayoutManager().getAbsolutWidth(swtIndex));
                        cmdl.setNumColumnIndex(i);
                        // cmdl.setNumOrder(i);
                        cmdl.setNumOrder(getTableWMClient().uiColumnIndexToModelColumnIndex(
                                getTable().getColumnOrder()[swtIndex]));
                        cmdl.setTxtColumn(attrLabel);
                        cmdl.setVisible(getTableWMClient().getColumn(colIndexInTableModel).isVisible());
                        cmdl.setResizeable(getTable().getColumn(swtIndex).getResizable());
                        cmdl.setDefMinWith(getTableLayoutManager().getMinWidth(swtIndex));
                        ctm.addColumnModel(cmdl);
                        IFmt formatter = getTableWMClient().getColumn(colIndexInTableModel).getFormatter();
                        String fmtStr = null;
                        if (formatter != null) {
                            if (formatter instanceof ABcdFmtDefault) {
                                int maxBeforeC = ((ABcdFmtDefault) formatter).getMaxBeforeC();
                                int maxAfterC = ((ABcdFmtDefault) formatter).getMaxAfterC();
                                fmtStr = maxBeforeC + "," + maxAfterC;
                            } else if (formatter instanceof ABcdFmtRange) {
                                int maxBeforeC = ((ABcdFmtRange) formatter).getMaxBeforeC();
                                int maxAfterC = ((ABcdFmtRange) formatter).getMaxAfterC();
                                fmtStr = maxBeforeC + "," + maxAfterC;
                            } else if (formatter instanceof ABcdFmtPattern) {
                                int maxBeforeC = ((ABcdFmtPattern) formatter).getMaxBeforeC();
                                int maxAfterC = ((ABcdFmtPattern) formatter).getMaxAfterC();
                                fmtStr = maxBeforeC + "," + maxAfterC;
                            } else if (formatter instanceof ADateFmtMediumSmart) {
                                fmtStr = ((ADateFmtMediumSmart) formatter).getPattern();
                            }
                            // TODO: custom formatters are not considered=>
                            // handle this use case
                            cmdl.setTxtFormatterPattern(fmtStr);
                        }
                        break;
                    }
                }

            }
        }
        return ctm;
    }

    public void setPagingWMClient(PagingWMClient pagingWMClient) {
        this.pagingWMClient = pagingWMClient;

    }

    public PagingWMClient getPagingWMClient() {
        return this.pagingWMClient;
    }

    public String getAppCode() {
        return appCode;
    }
    public void setAppCode(String appCode) {
        this.appCode = appCode;
    }

    /**
     * @see TableCustomizerComp#setFilter(String, int, boolean, List)
     */
    public void setFilter(String fqTableName, int columnId, boolean isAnd, List<CommonFilterModel> filters) {
        CommonTableModel commonModelFromTable = commonModelFromTable(fqTableName);
        RemoteCallClient newRemoteCall = newRemoteCall("setFilter");
        newRemoteCall.setParameter(0, commonModelFromTable);
        newRemoteCall.setParameter(1, Integer.valueOf(columnId));
        newRemoteCall.setParameter(2, Boolean.valueOf(isAnd));
        newRemoteCall.setParameter(3, filters);
        newRemoteCall.execute();
        getPagingWMClient().reload();
    }

    public void removeAllFilter() {
        newRemoteCall("removeAllFilter").execute();        
    }

    public CommonTableModel getTableModel() {
       return commonTableModel;
    }
    




}
