package org.openxma.addons.ui.table.customizer.xma.client;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import at.spardat.xma.component.EmbeddableComponent;
import at.spardat.xma.component.ComponentProperty;
import at.spardat.xma.session.XMASessionClient;

/**
 * This class implements the generic client part of TableCustomizerComp.
 * It provides the mapping between typeIds and classnames
 * of all pages in the component.
 *
 */
abstract public class TableCustomizerCompGen extends EmbeddableComponent implements ITableCustomizerComp {

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    protected String FQTableNameProperty;

    /** describes all component properties */
    private ComponentProperty[] propDes;

    /**
     * Initializes the TableCustomizerCompGen. This method is called
     * by the XMA-Runtime.
     *
     * @param session the XMASession, this TableCustomizerCompGen belongs to.
     */
    public TableCustomizerCompGen (XMASessionClient session) {
        super (session, false);
    }

    /** the unique typeId for the page TableCustomizerPage */
    public static final short TableCustomizerPageClass = 0;
    /** the unique typeId for the page FilterPage */
    public static final short FilterPageClass = 1;
    /** the unique typeId for the page FilterEmText */
    public static final short FilterEmTextClass = 2;
    /** the unique typeId for the page FilterEmDate */
    public static final short FilterEmDateClass = 3;
    /** the unique typeId for the page FilterEmNumber */
    public static final short FilterEmNumberClass = 4;
    /** the unique typeId for the page FilterEmDOM */
    public static final short FilterEmDOMClass = 5;

    /**
    * Array of the classnames of all pages in
    * this component. The typeIds are used as indices
    * into this array.
     */
    static final String[] modelClasses = {
        "TableCustomizerPage",
        "FilterPage",
        "FilterEmText",
        "FilterEmDate",
        "FilterEmNumber",
        "FilterEmDOM"
    };

    /**
    * Gets the short classname for the given typeId of
    * a page in this component. The short classname does
    * not contain the package.
    *
    * @param typeId the unique typeId
    * @return the short classname
     */
    static public String getShortModelClass(short typeId) {
        return modelClasses[typeId];
    }

    /**
    * Gets the full classname for the given typeId of
    * a page in this component.
    *
    * @param typeId the unique typeId
    * @return the full classname
     */
    public String getModelClass(short typeId) {
        return "org.openxma.addons.ui.table.customizer.xma.client."+modelClasses[typeId];
    }

    /**
     * Invokes the TableCustomizerComp. This method is a default
     * implementation, which invokes the first dialog page of the component.
     * Modeled init functions are called by this method.
     *
     * @param parentComposite a composite of the calling page. In most
     *     cases it will be a shell. It my be null.
     */
    public void invoke(Composite parentComposite) {
        TableCustomizerPage customizerPage;
        if(parentComposite instanceof Shell) {
            customizerPage = new TableCustomizerPage(this,(Shell)parentComposite);
        } else {
            customizerPage = new TableCustomizerPage(this);
        }
        customizerPage.invoke();
    }

    /**
     * Gives the URI of the help for this component
     * This URI is relative to the download-URI of the component
     * @return the relative URI of the help for this component
     */
    public String getHelpUri() {
        return "help/org/openxma/addons/ui/table/customizer/TableCustomizerComp.html";
    }

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    protected String getFQTableNameProperty() {
        return FQTableNameProperty;
    }

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    public void setFQTableNameProperty(String FQTableNameProperty) {
        String oldFQTableNameProperty = this.FQTableNameProperty;
        this.FQTableNameProperty = FQTableNameProperty;
        notifyPropertyChange("FQTableNameProperty",oldFQTableNameProperty,FQTableNameProperty);
    }

    /**
     * @return descriptors of the properties of this component
     */
    protected ComponentProperty[] getPropDes() {
        if(propDes==null) {
            propDes = new ComponentProperty[1];
            propDes[0]=new ComponentProperty("FQTableNameProperty",String.class,this,TableCustomizerCompGen.class);
        }
        return propDes;
    }

}
