package org.openxma.addons.ui.table.customizer.xma.server;

import at.spardat.xma.component.ComponentServer;
import at.spardat.xma.component.ComponentProperty;
import at.spardat.xma.session.XMASessionServer;

/**
 * This class implements the generic server part of TableCustomizerComp.
 * It provides the mapping between typeIds and classnames
 * of all pages in the component.
 *
 */
abstract public class TableCustomizerCompGen extends org.openxma.dsl.platform.xma.SpringComponentServer {

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    protected String FQTableNameProperty;

    /** describes all component properties */
    private ComponentProperty[] propDes;


    /**
     * Initializes the TableCustomizerCompGen. This method is called
     * by the XMA-Runtime.
     *
     * @param session the XMASession, this TableCustomizerCompGen belongs to.
     * @param id the unique id of this TableCustomizerComp.
     */
    public TableCustomizerCompGen (XMASessionServer session,short id) {
        super (session,false,id);
    }


    /** the unique typeId for the page TableCustomizerPage */
    public static final short TableCustomizerPageClass = 0;
    /** the unique typeId for the page FilterPage */
    public static final short FilterPageClass = 1;
    /** the unique typeId for the page FilterEmText */
    public static final short FilterEmTextClass = 2;
    /** the unique typeId for the page FilterEmDate */
    public static final short FilterEmDateClass = 3;
    /** the unique typeId for the page FilterEmNumber */
    public static final short FilterEmNumberClass = 4;
    /** the unique typeId for the page FilterEmDOM */
    public static final short FilterEmDOMClass = 5;

    /**
    * Array of the classnames of all pages in
    * this component. The typeIds are used as indices
    * into this array.
     */
    static final String[] modelClasses = {
        "TableCustomizerPage",
        "FilterPage",
        "FilterEmText",
        "FilterEmDate",
        "FilterEmNumber",
        "FilterEmDOM"
    };

    /**
    * Gets the short classname for the given typeId of
    * a page in this component. The short classname does
    * not contain the package.
    *
    * @param typeId the unique typeId
    * @return the short classname
     */
    static public String getShortModelClass(short typeId) {
        return modelClasses[typeId];
    }

    /**
    * Gets the full classname for the given typeId of
    * a page in this component.
    *
    * @param typeId the unique typeId
    * @return the full classname
     */
    public String getModelClass(short typeId) {
        return "org.openxma.addons.ui.table.customizer.xma.server."+modelClasses[typeId];
    }

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    protected String getFQTableNameProperty() {
        return FQTableNameProperty;
    }

    /**
     * full qualified table name - the name that is unique for a table within an xma application
     */
    public void setFQTableNameProperty(String FQTableNameProperty) {
        this.FQTableNameProperty = FQTableNameProperty;
    }

    /**
     * @return descriptors of the properties of this component
     */
    protected ComponentProperty[] getPropDes() {
        if(propDes==null) {
            propDes = new ComponentProperty[1];
            propDes[0]=new ComponentProperty("FQTableNameProperty",String.class,this,TableCustomizerCompGen.class);
        }
        return propDes;
    }


}
